grammar DSL;

options {
  language = Java;
  memoize = true;
  output=AST;
  ASTLabelType=SpcAST;
}

tokens {
  MAKE_REQ;
  NO_REC;
  EXPR;
  TEXT;
}
//***************************** Helper functions *******************************/
@parser::header {
  package dsl.antlr;
  
  import dsl.SpcAST;
}

@parser::members {
    
    /**
     * When a persing error occurs, this method is called. This overrides
     * the default behavior.
     * See: <a href="http://www.antlr.org/wiki/display/ANTLR3/Error+reporting+and+recovery">
     * the ANTLR error and recovery page</a> for more information.
     */
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e) 
    {
        String hdr = getErrorHeader(e);
        String msg = getErrorMessage(e, tokenNames);
        throw new RuntimeException("(" + hdr + ") " + msg, e);
    }
}

@lexer::header {
  package dsl.antlr;
}

/****************************** Parser grammer *******************************/
recommendation
    : 
    REC WS* longName WS* (stmts += statement)+ -> ^(REC[$longName.text] $stmts+)
    ;

statement
    :
      category
    | summary
    | description
    | scenario
    | mainAsset
    ;

category
    : 
    CATEGORY WS* longName (WS* | EOF) -> ^(CATEGORY[$longName.text])
    ;

summary
    :
    SUMMARY terminator -> ^(SUMMARY[$terminator.text.trim()])
    ;

description
    :
    DESC terminator -> ^(DESC[$terminator.text.trim()])
    ;

mainAsset
    :
    asset terminator -> ^(asset TEXT[$terminator.text.trim()])
    ;

asset
    :
    ASSET -> ^(ASSET[$ASSET.text])
    ;

scenario
    : 
    SCENARIO WS* 
    GIVEN WS* expressionList WS* 
    THEN WS* actionList (WS* | EOF)
    -> ^(SCENARIO expressionList actionList)
    ;

actionList
    :
    action (WS* actions += andAction)* -> ^(THEN action $actions*)
    ;

andAction
    :
    AND WS* action -> action
    ;

action
    :
      addConfidence
    | removeConfidence
    | STOP
    | MAKE WS* REQUIRED -> MAKE_REQ
    | DO WS* NOT WS* RECWORD -> NO_REC
    | addAsset
    ;
        
addConfidence
    :
    ADD WS* INTEGER WS* CONFIDENCE -> ^(ADD[$INTEGER.text])
    ;
    
removeConfidence
    :
    REMOVE WS* INTEGER WS* CONFIDENCE -> ^(REMOVE[$INTEGER.text])
    ;

addAsset
    :
    ADD WS* inlineAsset -> inlineAsset
    ;

inlineAsset
    :
    LBRACE WS* asset notRBrace RBRACE -> ^(asset TEXT[$notRBrace.text.trim()])
    ;

notRBrace
    :
    ~(RBRACE)+
    ;

expressionList
    :
    expression (WS* exps += logicalExpression)* -> ^(GIVEN expression $exps*)
    ;

expression
    :
    name WS* operator (WS* ops += operatorList)* -> ^(EXPR[$name.text] operator $ops*)
    ;
    
operatorList
    :
    OR  WS* operator -> operator
    ;

logicalExpression
    :
    AND WS* expression -> expression
    ;

operator
    : 
      singleOperator WS* value -> ^(singleOperator value)
    | doubleOperator WS* val1=value WS* AND WS* val2=value -> ^(doubleOperator $val1 $val2)
    ;
          
singleOperator
    : 
      equal -> EQL
    | notEqual -> NEQL
    | greaterThan -> GT
    | lessThan -> LT
    | greaterThanEqual -> GTE
    | lessThanEqual -> LTE
    ;

doubleOperator
    :
    between -> BETWEEN
    ;

equal
    :
      IS
    | (IS WS*)? (EQL| (EQLWORD WS* TO))
    | EQLS
    ;
        
notEqual
    :
      (IS WS*)? (NEQL | (NOT WS* EQLWORD WS* TO))
    | IS WS* NOT
    ;

greaterThan
    :
    (IS WS*)? (GT | (GREATER WS* THAN))
    ;

lessThan
    :
    (IS WS*)? (LT | (LESS WS* THAN))
    ;

greaterThanEqual
    :
    (IS WS*)? (GTE | (GREATER WS* THAN WS* OR WS* EQLWORD WS* TO))
    ;

lessThanEqual
    :
    (IS WS*)? (LTE | (LESS WS* THAN WS* OR WS* EQLWORD WS* TO))
    ;

between
    :
    (IS WS*)? BETWEEN
    ;


value
    :
      name -> ^(TEXT[$name.text])
    | INTEGER
    | booleanConst
    | STRING -> ^(STRING[$STRING.text.replaceAll("\"","")]);

booleanConst
    : 
      TRUE
    | FALSE;

terminator
    :
    ~(CATEGORY|DESC|SCENARIO|SUMMARY|ASSET|EOF)+
    ;

name
    :
    WORD (WS* WORD)*
    ;

longName
    :
    word (WS* word)*
    ;
    
word
    :
     GIVEN | THEN | ADD | REMOVE | CONFIDENCE 
    | STOP | MAKE | REQUIRED | DO | NOT | RECWORD
    | AND | IS | THAN | OR | EQL | EQLS | EQLWORD
    | NEQL | TO | GREATER | GT | LESS | LT | GTE | LTE
    | BETWEEN | TRUE | FALSE | INTEGER | WORD 
    ;

/****************************** Scanner grammer ******************************/
//originally was trying to do too much in lexer, now reduced to just reserved words
//plus the definition of value types

//whitespace is not ignored
WS: (' ' | '\r' | '\t' | '\f' | '\n')+ ;

//Main heading keywords
REC : 'Recommendation:';
CATEGORY: 'Category:';
SUMMARY: 'Summary:';
DESC: 'Description:';
SCENARIO: 'Scenario:';
ASSET: (('Link'|'Text'|'Book'|'Online' WS* 'Image') WS* 'Asset:');

COLON : ':';

//Scenario keywords
GIVEN: 'Given';
THEN: 'Then';
ADD: ('A'|'a')'dd';
REMOVE: ('R'|'r')'emove';
CONFIDENCE: ('C'|'c')'onfidence';
STOP: ('S'|'s')'top';
MAKE: ('M'|'m')'ake';
REQUIRED: ('R'|'r')'equired';
DO: ('D'|'d')'o';
NOT : ('N'|'n')'ot';
RECWORD : ('R'|'r')'ecommend';
AND: ('A'|'a')'nd';

//operator keywords (grouped by operator)
IS: 'is';
THAN : 'than';
OR: 'or';

EQL : '=';
EQLS : 'equals';
EQLWORD : 'equal';
TO: 'to';

NEQL: '!=';

GREATER: 'greater';
GT: 'gt' | '>';

LESS: 'less';
LT: 'lt' | '<';

GTE: 'gte' | '>=';
LTE: 'lte' | '<=';

BETWEEN: 'between';

//value keywords
TRUE: 'true';
FALSE: 'false';

LBRACE : '{';
RBRACE : '}';

INTEGER: DIGIT+ ;
WORD : (LETTER|DIGIT|'_'|'-'|'\'')+;
OTHER: .;

STRING: '"' (options {greedy=false;} : . )* '"';
      
fragment
LETTER  :
        ('a'..'z'|'A'..'Z')
        ;
        
fragment
DIGIT   :
        ('0'..'9')
        ;
