/*******************************************************************************
 * This file is part of Jake Todaro and Pragathi Balasubramanian's MQP.
 * Software Process Configurator MQP
 * Advised by Professor Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jake Todaro
 *******************************************************************************/
package configurator;

import java.util.LinkedList;
import java.util.List;

import asset.Asset;

import com.db4o.ObjectContainer;
import com.db4o.ObjectSet;

import database.AssetDatabase;
import dialog.AnswerSet;

/**
 * This asset selecting strategy chooses the n most relevant assets that are
 * relevant to the users responses. Every asset in the database is scored based on
 * relevant it is. The score for each asset is determined by the number of tags in
 * the answer set that match with the assets tags. For example, if an asset has tags
 * {"a", "b"}, and the answer set contains answers with tags {"a"}, {"a", "b"}, and {"b", "c"}
 * the asset would score 4, because there are 4 tags in the set of answer's tags that
 * match with tags in the assets tag list
 * 
 * @author jake todaro
 *
 */

public class FindMostRelevant implements AssetMatcher{
	
	private int numberOfassetsToReturn;
	
	/**
	 * constructor iniializes the fields
	 * 
	 * @param n - the number of relevant assets to return
	 */
	public FindMostRelevant(int n){
		numberOfassetsToReturn = n;
	}

	@Override
	public List<Asset> findRelevantAssets(AnswerSet answerSet) {
		
		/* get all of the answers tags
			IMPORTANT! includes duplicates!
			duplicates are needed for this to score the assets accurately
		 */
		List<String> allAnswerTags = answerSet.getAllAnswersAssetTags();
	
		ObjectContainer dboc = AssetDatabase.getObjectContainer();
		ObjectSet<Asset> assets = dboc.queryByExample(Asset.class);
		
		int[] assetMatches = new int[assets.size()];
		
		//score the assets
		for(int i = 0; i < assets.size(); i++){
			Asset asset = assets.get(i);

			for(int j = 0; j < allAnswerTags.size(); j++){
				if(asset.getTags().contains(allAnswerTags.get(j)))
					assetMatches[i]++;
			}
		}
		
		List<Asset> relevantAssets = new LinkedList<Asset>();
		
		//select the n highest scoring assets
		int max = 0, maxIndex = 0;
		for(int i = 0; i < numberOfassetsToReturn; i++){
			for(int j = 0; j < assetMatches.length; j++){
				if(assetMatches[j] > max){
					max = assetMatches[j];
					maxIndex = j;
				}
			}
			relevantAssets.add(assets.get(maxIndex));
			assetMatches[maxIndex] = -1;
			max = 0;
		}
		
		return relevantAssets;
	}
}
