/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package app;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JTextPane;

import value.valueData.ValueType;

import attribute.AttributeDescription;
import attribute.AttributeReference;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

import database.Database;
import javax.swing.JButton;

/**
 * A user interface for adding or editing an attribute in the database
 * 
 * @author Wesley Ripley, Adam Talbot
 * @version Feb 28, 2013
 */
@SuppressWarnings("serial")
public class AttributeEditor extends JDialog implements ActionListener{

	private final Database db;
	private final AttributeReference attrRef;
	private boolean edit = false;
	private AttributeDescription attr = null;
	
	//fields that we need for later
	private JTextField nameField;
	private JComboBox typeBox;
	private JTextPane descField;
	private JButton saveBtn;
	private JButton saveAndNewBtn;
	private JButton closeBtn;
	
	/**
	 * Constructor for AttributeEditor used when adding a new attribute
	 * @param db
	 * 		The database to add the attribute to
	 * @param owner
	 * 		The frame that is creating this frame
	 * @param modal
	 * 		Whether this frame should be opened as a modal or not.
	 */
	public AttributeEditor(Database db, Frame owner,boolean modal) {
		super(owner,"Attribute Editor",modal);
		this.db = db;
		attrRef = db.getAttributeReference();
		initDialog();
		setVisible(true);
	}
	
	/**
	 * Constructor for AttributeEditor used when editing an attribute
	 * @param db
	 * 		The database the hold the attribute to edit
	 * @param owner
	 * 		The frame that is creating this frame
	 * @param modal
	 * 		Whether this frame should be opened as a modal or not.
	 * @param attrName
	 * 		The name of the attribute to edit
	 */
	public AttributeEditor(Database db, Frame owner,boolean modal, String attrName) {
		super(owner,"Attribute Editor",modal);
		edit = true;
		this.db = db;
		attrRef = db.getAttributeReference();
		attr = attrRef.getAttr(attrName);
		//initialize UI
		initDialog();
		saveAndNewBtn.setEnabled(false);
		saveAndNewBtn.setVisible(false);
		nameField.setText(attr.getName());
		typeBox.setSelectedItem(attr.getType());
		descField.setText(attr.getDescription());
		setVisible(true);
	}
	
	/**
	 * Method that initializes the UI components
	 * Created using WindowBuilder Pro
	 */
	private void initDialog() {
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setSize(325,318);
		
		getContentPane().setLayout(new FormLayout(new ColumnSpec[] {
				FormFactory.RELATED_GAP_COLSPEC,
				FormFactory.DEFAULT_COLSPEC,
				FormFactory.RELATED_GAP_COLSPEC,
				ColumnSpec.decode("default:grow"),
				FormFactory.RELATED_GAP_COLSPEC,
				ColumnSpec.decode("default:grow"),
				FormFactory.RELATED_GAP_COLSPEC,
				FormFactory.DEFAULT_COLSPEC,},
			new RowSpec[] {
				FormFactory.RELATED_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC,
				FormFactory.RELATED_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC,
				FormFactory.RELATED_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC,
				FormFactory.RELATED_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC,
				FormFactory.RELATED_GAP_ROWSPEC,
				RowSpec.decode("max(78dlu;default):grow"),
				FormFactory.RELATED_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC,}));
		
		JLabel lblName = new JLabel("Name:");
		getContentPane().add(lblName, "2, 4, right, default");
		
		nameField = new JTextField();
		getContentPane().add(nameField, "4, 4, 3, 1, fill, default");
		nameField.setColumns(10);
		
		JLabel lblType = new JLabel("Type:");
		getContentPane().add(lblType, "2, 6, right, default");
		
		typeBox = new JComboBox(ValueType.values());
		getContentPane().add(typeBox, "4, 6, 3, 1, fill, default");
		
		JLabel lblDescription = new JLabel("Description:");
		getContentPane().add(lblDescription, "2, 8, right, default");
		
		descField = new JTextPane();
		getContentPane().add(descField, "4, 8, 3, 3, fill, fill");
		
		saveAndNewBtn = new JButton("Save and New");
		saveAndNewBtn.addActionListener(this);
		getContentPane().add(saveAndNewBtn, "2, 12");
		
		saveBtn = new JButton("Save");
		saveBtn.addActionListener(this);
		getContentPane().add(saveBtn, "4, 12");
		
		closeBtn = new JButton("Close");
		closeBtn.addActionListener(this);
		getContentPane().add(closeBtn, "6, 12");
	}

	/**
	 * Function called when any button is pressed.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == closeBtn){
			//if close clicked, then close
			dispose();
		} else if(e.getSource() == saveBtn || e.getSource() == saveAndNewBtn) {
			//if save or save and new clicked
			//build attribute description from input
			AttributeDescription attrDesc = new AttributeDescription(nameField.getText(),(ValueType)typeBox.getSelectedItem(),descField.getText());
			if(edit) {
				attrRef.removeAttribute(attr.getName());
			}
			//save to attribute reference
			attrRef.addAttribute(attrDesc);
			if(e.getSource() == saveAndNewBtn) {
				//reset UI
				edit = false;
				nameField.setText("");
				typeBox.setSelectedIndex(0);
				descField.setText("");
			} else {
				//update attribute reference in database and close
				db.store(attrRef);
				dispose();
			}
		}
	}
	

}
