/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package app;

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import org.antlr.runtime.RecognitionException;

import recommendation.Recommendation;
import database.Database;
import dsl.DSLLoader;

/**
 * A user interface for loading and editing a Recommendation in the database
 * 
 * @author Wesley Ripley, Adam Talbot
 * @version Feb 28, 2013
 */
@SuppressWarnings("serial")
public class RecommendationLoader extends JDialog implements ActionListener{

	private final Database db;
	private JTextArea statusText;
	private final boolean edit;
	private File toEdit = null;
	private File lastDirectory = null;
	
	/**
	 * Constructor for RecommendationLoader when adding a Recommendation
	 * 
	 * @param db
	 * 		The database to add to
	 * @param owner
	 * 		The Frame that is creating this one
	 * @param modal
	 * 		Whether this frame should be opened modally
	 */
	public RecommendationLoader(Database db, Frame owner,boolean modal) {
		super(owner,"Recommendation Loader",modal);
		this.db = db;
		this.edit = false;
		initUI();
	}
	
	/**
	 * Constructor for RecommendationLoader when editing a Recommendation
	 * 
	 * @param db
	 * 		The database where the recommendation is
	 * @param owner
	 * 		The Frame that is creating this one
	 * @param modal
	 * 		Whether this frame should be opened modally
	 * @param toEdit
	 * 		The recommendation file that is being edited
	 */
	public RecommendationLoader(Database db, Frame owner,boolean modal,File toEdit) {
		super(owner,"Recommendation Loader",modal);
		this.db = db;
		this.edit = true;
		this.toEdit = toEdit;
		initUI();
		
	}
	
	/**
	 * Initializes UI Components
	 */
	private void initUI() {
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setSize(448,240);
		
		Container content = getContentPane();
		FlowLayout layout = new FlowLayout();
		content.setLayout(layout);
		
		String loadBttnName;
		if(edit) {
			loadBttnName = "Reload Recommendation";
		} else {
			loadBttnName = "Load Recommendation";
		}
		JButton loadButton = new JButton(loadBttnName);
		
		loadButton.addActionListener(this);
		
		content.add(loadButton);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		content.add(scrollPane);
		String message;
		if(edit) {
			message = "Edit the recommendation opened (do not move it) and click \"Reload Recommendation\" when done";
		} else {
			message = "Click \"Load Recommendation\" to load a recommendation file into the system\n";
		}
		statusText = new JTextArea(message,10,35);
		statusText.setEditable(false);
		scrollPane.setViewportView(statusText);
		statusText.setWrapStyleWord(true);
		statusText.setLineWrap(true);
		
		setVisible(true);
	}

	/**
	 * Method called when any button is clicked
	 */
	@Override
	public void actionPerformed(ActionEvent arg0) {
		File[] recFiles = null;
		if(edit) {
			//if editing we already know which file to load.
			recFiles = new File[1];
			recFiles[0] = toEdit;
		} else {
			//give user open dialog to choose recommendation file
			JFileChooser fc = new JFileChooser();
			fc.setMultiSelectionEnabled(true);
			fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
			if(lastDirectory != null) {
				//start in last directory used for convenience
				fc.setCurrentDirectory(lastDirectory);
			}
			int returnValue = fc.showOpenDialog(this);
			if(returnValue == JFileChooser.APPROVE_OPTION) {
				recFiles = fc.getSelectedFiles();
				if(recFiles != null && recFiles.length > 0) {
					//keep track of last directory used
					lastDirectory = recFiles[0].getParentFile();
				}
			}
		}
		if(recFiles != null) {
			//load each file into the database giving appropriate errors
			for(File recFile : recFiles) {
				try {
					Recommendation newRec = DSLLoader.loadRecommendation(new FileReader(recFile), db);
					appendStatusText("Recommendation "+ newRec.getName() +" sucessfully loaded!");
				} catch (FileNotFoundException e) {
					appendStatusText("File "+recFile.getAbsolutePath() +" was not found");
				} catch (IOException e) {
					appendStatusText("Error reading file: "+e.getMessage());
				} catch (RecognitionException e) {
					appendStatusText("Syntax error in file: "+e.getMessage());
				} catch(RuntimeException e) {
					appendStatusText("Syntax error in file: "+e.getMessage());
				}
			}
		}
	}
	
	/**
	 * Appends text to the status box
	 * @param text
	 * 		The new text to be appended
	 */
	private void appendStatusText(String text) {
		statusText.append("\n"+text+"\n");
	}

}
