
package attribute;

import value.BooleanValue;
import value.IntegerValue;
import value.StringValue;
import value.Value;
import value.valueData.ValueType;
import static dsl.antlr.DSLRecCreator.*;

/**
 * A class to build a Attribute piece by piece
 * 
 * Used in creating attributes from the DSL
 * 
 * @author Wesley Ripley, Adam Talbot
 * @version Feb 28, 2013
 */
public class AttributeBuilder {
	
	private String name = null;
	private Value value = null;
	private ValueType type = null;
	
	/**
	 * Set the name of the attribute.
	 * 
	 * Must be the same name as an attribute in the AttributeReference
	 * @param name
	 * 		The name
	 */
	public void setName(String name) {
		this.name = name;
		//TODO: need to pull AttributeDescription for database to validate type
	}
	
	/**
	 * Set the type of this attribute
	 * 
	 * If name is set, must match the type specified in the AttributeReference
	 * @param type
	 * 		The type
	 */
	public void setType(ValueType type) {
		this.type = type;
		//TODO: need type validation with attribute reference
	}
	
	/**
	 * Add a value to the attribute
	 * 
	 * The type must be set in order to add attributes
	 * 
	 * @param operator
	 * 		The operator token from the DSL. See dsl.antlr.DSLRecCreator for what values mean which operators
	 * @param val1
	 * 		The first value string. String must be able to be parsed into the type set. For single operators this is the only value.
	 * @param val2
	 * 		The second value for double operators or null for single operators. String must be able to be parsed into the type set.
	 */
	public void addValue(int operator, String val1, String val2) {
		if(type == null) {
			throw new IllegalStateException("Cannot add a value without setting the type");
		}
		switch(type) {
			case BOOLEAN:
				//booleans can only have one value
				if(value != null) {
					throw new IllegalArgumentException("Cannot have multiple operators on boolean type attribute");
				}
				BooleanValue newValue = (BooleanValue) type.makeValue(val1);
				switch(operator) {
				case EQL:
					value = newValue;
					break;
				case NEQL:
					value = new BooleanValue(!newValue.getContents());
					break;
				default:
					throw new IllegalArgumentException("Only equal and not equal allowed for boolean");
				}
				
				break;
			case STRING:
				if(value == null) {
					value = new StringValue();
				}
				StringValue stringVal = (StringValue)value;
				switch(operator) {
				case EQL:
					stringVal.addString(val1);
					break;
				case NEQL:
					stringVal.addNotString(val1);
					break;
				default:
					throw new IllegalArgumentException("Only equal and not equal allowed for strings");
				}
				
				break;
			case INT:
				if(value == null) {
					value = new IntegerValue();
				}
				try {
					int num1 = Integer.parseInt(val1);
					IntegerValue intValue = (IntegerValue)value;
					switch(operator) {
					case EQL:
						intValue.addSingle(num1);
						break;
					case BETWEEN:
						intValue.addBetween(num1, Integer.parseInt(val2));
						break;
					case NEQL:
						intValue.addNot(num1);
						break;
					case GT:
					case GTE:
						intValue.addGreaterThan(num1, operator == GTE);
						break;
					case LT:
					case LTE:
						intValue.addLessThan(num1, operator == LTE);
						break;
					default:
						throw new IllegalArgumentException("Invalid operator provided");
					}
				} catch (NumberFormatException e) {
					throw new IllegalArgumentException("Integer values must be an integer");
				}
				break;
			default:
				throw new IllegalArgumentException("Invalid Value Type");
			}
	 }
	 
	 public Attribute buildAttribute() {
		 if(name == null || type == null || value == null) {
			 throw new IllegalStateException("Must set name, type, and value before building attribute");
		 }
		 //TODO: This should call AttributeDescription.makeAttribute
		 return new Attribute(name,value);
	 }

}
