/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package attribute;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * AttributeReference is a map of Attribute names to AttributeDescriptions.
 * It is for user reference so that a user adding recommendations can know
 * what Attributes already exist.
 * @author Wesley Ripley, Adam Talbot
 * @version 2/20/13
 */
public class AttributeReference {
	
	private final Map<String, AttributeDescription> availableAttrs;
	
	/**
	 * Constructor for an AttributeReference.  Used for initialization.
	 */
	public AttributeReference() {
		availableAttrs = new HashMap<String, AttributeDescription>();
	}
	
	/**
	 * Gets the AttributeDescription that matches a given Attribute name
	 * @param name The name of any Attribute
	 * @return AttributeDescription related to the given Attribute
	 */
	public AttributeDescription getAttr(String name) {
		if(name == null || name.equals("")) {
			return null;
		}
		return availableAttrs.get(name);
	}
	
	/**
	 * Gets all of the Attributes that exist.
	 * @return a Collection of AttributeDescriptions
	 */
	public Collection<AttributeDescription> getAllAvaiableAttrs() {
		return availableAttrs.values();
	}
	
	/**
	 * Searches the attributes for any names that contain the search string
	 * @param search String containing what to search for
	 * @return Collection of AttributeDescriptions with names that contain the search string
	 */
	public Collection<AttributeDescription> searchForAttr(String search) {
		if(search == null || search.equals("")) {
			return null;
		}
		//very simple search for now, assumes search is one word
		List<AttributeDescription> foundAttrs = new ArrayList<AttributeDescription>();
		for(AttributeDescription attr : availableAttrs.values()) {
			if(attr.getName().contains(search) ||
					attr.getDescription().contains(search)) {
				foundAttrs.add(attr);
			}
		}
		return foundAttrs;
	}
	
	/**
	 * Adds Attribute information to the AttributeReference
	 * @param attrDesc AttributeDescription to be added to the Reference,
	 * and also provides the name of the attribute
	 */
	public void addAttribute(AttributeDescription attrDesc) {
		availableAttrs.put(attrDesc.getName(), attrDesc);
	}
	
	/**
	 * Removes the specified Attribute, if it exists
	 * @param name the name of the Attribute to remove
	 */
	public void removeAttribute(String name) {
		availableAttrs.remove(name);
	}
	
}
