/*******************************************************************************
 * This file is part of Jake Todaro and Pragathi Balasubramanian's MQP.
 * Software Process Configurator MQP
 * Advised by Professor Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jake Todaro
 *******************************************************************************/
package dialog;

import value.Value;
import attribute.Attribute;
import attribute.AttributeDescription;
import booleanExpression.BooleanValue;

/**
 * This class represents a possible answer to a Question. Answers have tags
 * associated with them that are used to represent information about a users
 * project that can be assumed true if the user selects this answer.
 * 
 * An answer is also sufficient as an asking requirement.
 * 
 * @author jake todaro
 * 
 * @version 12/09/11
 */

public class Answer implements BooleanValue {

	private final int answerNum;
	private final String answerText;
	private final Value attrValue;
	private Question parentQuestion;
	private boolean isSelected;
	private Attribute attr = null;

	/**
	 * constructor initializes the fields
	 * 
	 * @param answerNum
	 *            - the number of this Answer, this should be 1 + the index of
	 *            the Answer in its parent Question
	 * @param text
	 *            - the text that represents this Answer
	 */
	public Answer(int answerNum, String text, Value attrValue) {
		this.answerNum = answerNum;
		answerText = text;
		parentQuestion = null;
		isSelected = false;
		this.attrValue = attrValue;
	}
	
	public Answer(int answerNum, String text) {
		this(answerNum,text,null);
	}
	
	public void setParentQuestion(Question parent){
		parentQuestion = parent;
	}

	/**
	 * return the answer text
	 * 
	 * @return the answer text
	 */
	public String getAnswerText() {
		return answerText;
	}

	/** return the parent Question of this Answer
	 * 
	 * @return the parent Question of this Answer
	 */
	public Question getParentQuestion() {
		return parentQuestion;
	}

	/** return the answer number of this Answer, this number is 1 + the index
	 * of the Answer's position in the parent Question's Answer List
	 * 
	 * @return the answer number of this Answer
	 */
	public int getAnswerNum() {
		return answerNum;
	}
	
	/** this method handles selecting this Answer */
	public void select(){
		if(!isSelected){
			isSelected = true;
		
			AttributeDescription attrDesc = parentQuestion.getAttributeDescription();
			if(attrDesc != null && attrValue != null && attr == null) {
				attr = attrDesc.makeAttribute(attrValue);
				Dialog.addAttribute(attr);
			}
		}
	}
	
	/** this method handles deselecting this Answer */
	public void deselect(){
		if(isSelected){
			isSelected = false;
			
			if(attr != null) {
				Dialog.removeAttribute(attr);
			}
		}
	}

	@Override
	public String toString() {
		
		String result = "SubDialog "
				+ parentQuestion.getParentSubDialog().getTitle()
				+ ", Question " + parentQuestion.getId() + ", Answer "
				+ answerNum + ": " + answerText + "\n";

		if(attr != null) {
			result += "\t\t\t" + "Means " + attr.getName() + " = "+ attr.getValue().toString() + "\n";
		}
		return result;
	}

	@Override
	public boolean value() {
		return isSelected;
	}
}
