/*******************************************************************************
 * This file is part of Jake Todaro and Pragathi Balasubramanian's MQP.
 * Software Process Configurator MQP
 * Advised by Professor Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jake Todaro
 *******************************************************************************/

package dialog;

import java.util.LinkedList;
import java.util.List;

import attribute.AttributeDescription;
import booleanExpression.BooleanValue;
import booleanExpression.TrueBooleanAtom;


/** This class represents a question that is in a Dialog. 
 * 
 * @author jake todaro
 * 
 * @version 12/09/11
 */
public class Question{
	
	private String id;
	private final String questionStatement;
	private final List<Answer> answerList;
	private SubDialog parentSubDialog;
	private BooleanValue trigger;
	private final AttributeDescription attrDesc;
	
	
	/**
	 * constructor initializes fields
	 * 
	 * @param text - the question
	 * @param attrDesc - the attribute description of the attribute this question is asking about
	 */	
	public Question(String text, AttributeDescription attrDesc){
		id = null;
		questionStatement = text;
		parentSubDialog = null;
		answerList = new LinkedList<Answer>();
		trigger = null;
		this.attrDesc = attrDesc;
	}
	
	public Question(String text) {
		this(text,null);
	}
	
	public void setID(String id){
		this.id = id;
	}
	
	public void setTrigger(BooleanValue trigger){
		this.trigger = trigger;
	}
	
	public SubDialog getParentSubDialog(){
		return parentSubDialog;
	}
	
	public void setParentSubDialog(SubDialog parent){
		parentSubDialog = parent;
	}
	
	/**
	 * add a possible answer to the question
	 * 
	 * @param answer - the answer to add
	 */
	public void addAnswer(Answer answer){
		answer.setParentQuestion(this);
		answerList.add(answer);
	}
	
	/**
	 * return the question statement
	 * 
	 * @return the question statement
	 */
	public String getQuestionStatement(){
		return questionStatement;
	}
	
	/**
	 * return the list of possible answers
	 * 
	 * @return the list of possible answers
	 */
	public List<Answer> getAnswerList(){
		return answerList;
	}

	/** return the Trigger for this Question
	 * 
	 * @return the Trigger for this Question
	 */
	public BooleanValue getTrigger(){
		if(trigger == null){
			return new TrueBooleanAtom();
		}
		return trigger;
	}
	
	String getId() {
		return id;
	}
	
	public AttributeDescription getAttributeDescription() {
		return attrDesc;
	}
	
	@Override
	public boolean equals(Object input){
		if(this == input){
			return true;
		}
		
		return input instanceof Question &&
			id.equals(((Question) input).getId()) &&
			questionStatement.equals(((Question) input).getQuestionStatement()) &&
			answerList.equals(((Question) input).getAnswerList()) &&
			parentSubDialog == ((Question) input).getParentSubDialog();
	}
	
	@Override
	public String toString(){
		
		String result = "Question " + id + ": " + questionStatement + "\n";

		for(int i = 0; i < answerList.size(); i++){
			result += "\t\t" + answerList.get(i).toString();
		}
		
		result += "\n\t\tTrigger\n\t\t";
		
		if (trigger != null){
			result += trigger.toString();
		}
		
		return result;
	}
}
