/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package dsl;

import java.util.*;
import org.antlr.runtime.*;
import org.antlr.runtime.tree.CommonTree;

/**
 * This class defines the AST tree node for the compiler.
 * The tree node has added fields that we need to do semantic analysis
 * and IR creation for the dsl.
 * 
 * @author Wesley Ripley, gpollice
 * @version 11/25/12
 */
public class SpcAST extends CommonTree
{
	
	/**
	 * Default constructor.
	 */
	public SpcAST()
	{
		super();
	}

	/**
	 * <p>
	 * This constructor is needed for the tree walkers. It is needed for the 
	 * dupNode method.
	 * </p><p>
	 * After the duplication, any additional fields must be copied to the
	 * new node from the parameter.
	 * </p>
	 * 
	 * @param tree the tree from which the token will be used to create the new
	 * 	tree node.
	 * @see org.antlr.runtime.tree.CommonTree#CommonTree(CommonTree)
	 */
	public SpcAST(SpcAST tree)
	{
		super(tree);
	}

	/**
	 * <p>
	 * Simple constructor that takes a token.
	 * </p><p>
	 * This constructor also initializes any fields for the DijkstraTree  node
	 * with default values.
	 * </p>
	 * @param token the token used for initialization
	 * @see org.antlr.runtime.tree.CommonTree#CommonTree(Token)
	 */
	public SpcAST(Token t)
	{
		super(t);
	}

	/* (non-Javadoc)
	 * @see org.antlr.runtime.tree.CommonTree#dupNode()
	 */
	@Override
	public SpcAST dupNode()
	{
		return new SpcAST(this);
	}


	/* ******************************************************************************************
	 * The following methods are included for testing purposes. They are not used for any parsing
	 * purpose.
	 */
	
	public SpcAST(Token t, SpcAST left, SpcAST right)
	{
		super(t);
		this.addChild(left);
		this.addChild(right);
	}
	
	public SpcAST(Token t, List<SpcAST>children)
	{
		super(t);
		this.addChildren(children);
	}
	
	public SpcAST(int tokenType, String tokenText, SpcAST left, SpcAST right)
	{
		super(new CommonToken(tokenType, tokenText));
		this.addChild(left);
		this.addChild(right);
	}
	
	public SpcAST(int tokenType, String tokenText, List<SpcAST>children)
	{
		super(new CommonToken(tokenType, tokenText));
		this.addChildren(children);
	}
	
	public SpcAST(int tokenType, String tokenText)
	{
		this(tokenType, tokenText, null, null);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object tree)
	{
		SpcAST t = (SpcAST)tree;
		boolean isEqual = true;
		isEqual = (isEqual && this.getType() == t.getType());
		isEqual = isEqual && this.getText().equals(t.getText());
		isEqual = (isEqual && this.getChildCount() == t.getChildCount());
		for (int i = 0; (isEqual && i < this.getChildCount()); i++) {
			isEqual = isEqual && this.getChild(i).equals(t.getChild(i));
		}
		return isEqual;
	}

	
	// Utility functions
	/**
	 * @return the left child (binary tree terminology)
	 */
	public SpcAST leftChild()
	{
		return (SpcAST)getChild(0);
	}
	
	/**
	 * @return the right child (binary tree terminology)
	 */
	public SpcAST rightChild()
	{
		return (SpcAST)getChild(1);
	}
	
	/**
	 *  @returns a structured, annotated string representation of the tree.
	 */
	public String toFormattedString()
	{
		return constructTreeString(this, "", false);
	}
	
	public void showFormattedString() {
		System.out.println(toFormattedString());
	}
	
	public String toExtendedFormattedString()
	{
		return constructTreeString(this, "", true);
	}
	
	public void showExtendedFormattedString()
	{
		System.out.println(toExtendedFormattedString());
	}
	
	static private final String INDENT_STRING = "  ";
	
	@SuppressWarnings("unchecked")
	private String constructTreeString(SpcAST node, String indent, boolean moreDetail)
	{
		StringBuilder sb = new StringBuilder();
		if (moreDetail) {
			sb.append("(" + this.hashCode() + ") ");
		}
		sb.append(indent + node.getType() + " [" +
				node.getText() + "]\n");
		if (node.getChildCount() > 0) {
			for (SpcAST n : (List<SpcAST>)node.getChildren()) {
				sb.append(n.constructTreeString(n, indent + INDENT_STRING, moreDetail));
			}
		}
		return sb.toString();
	}
}
