/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package recommendation;

import java.util.ArrayList;
import java.util.List;

import recommendation.asset.Asset;

/**
 * This is the simplest implementation of a Recommendation.
 * @author Wesley Ripley, Adam Talbot
 * @version 10/03/12
 */
public class BasicRecommendation implements Recommendation {

	private final String name;
	private final List<Scenario> scenarios;
	private final List<Asset> assets;
	private final String description;
	private final String summary;
	private final List<String> categories;
	
	/**
	 * Constructor for a basic recommendation
	 * @param name The name of this Recommendation
	 * @param scenarios A list of Scenarios for which this Recommendation applies
	 * @param assets A list of Assets that this Recommendation provides
	 * @param description A String to be displayed to the user
	 * @param summary A short String to summarize this Recommendation
	 * @param categories A list of Strings that categorize this Recommendation 
	 */
	public BasicRecommendation(String name, List<Scenario> scenarios, List<Asset> assets, String description, String summary,List<String> categories)
	{
		this.name = name;
		this.scenarios = scenarios;
		this.assets = assets;
		this.description = description;
		this.categories = categories;
		this.summary = summary;
	}
	
	/**
	 * Constructor for a basic recommendation with no Summary or Categories.
	 * @param name The name of this Recommendation
	 * @param scenarios A list of Scenarios for which this Recommendation applies
	 * @param assets A list of Assets that this Recommendation provides
	 * @param description A String to be displayed to the user
	 */
	public BasicRecommendation(String name, List<Scenario> scenarios, List<Asset> assets, String description) {
		this(name,scenarios,assets,description,"",new ArrayList<String>());
	}
	
	/**
	 * Constructor for a basic recommendation with no Assets, Summary, or Categories.
	 * @param name
	 * @param scenarios
	 * @param description
	 */
	public BasicRecommendation(String name, List<Scenario> scenarios, String description) {
		this(name,scenarios,new ArrayList<Asset>(),description);
	}
	
	@Override
	public RecommendationResult evaluateScenario(Constraints constraints) {
		StringBuilder explaination = new StringBuilder();
		int confidence = 0;
		List<Asset> recommendedAssets = new ArrayList<Asset>(assets);
		boolean required = false;
		for(Scenario scenario : scenarios)
		{
			//TODO: When we have code to go from scenario -> dsl make all explainations more specific more specific
			if (scenario.evaluate(constraints)) {
				int weight = scenario.getWeight();
				explaination.append("Scenario added ").append(weight).append(" confidence\n");
				confidence += weight;
				
				if(scenario.isRequired()) {
					required = true;
				}
				if(scenario.isNotRecommended()) {
					confidence = 0;
					explaination.append("Specifically not recommended because of the one scenario\n"); 
					break;
				}
				List<Asset> scenarioAssets = scenario.getAssets();
				if(scenarioAssets != null) {
					recommendedAssets.addAll(scenarioAssets);
				}
				if(scenario.shouldStop()) {
					explaination.append("Stop caused by scenario\n");
					break;
				}
			}
		}
		return new RecommendationResult(confidence,explaination.toString(),recommendedAssets,required,this);
	}

	@Override
	public String getDescription() {
		return description;
	}
	
	@Override
	public String getName() {
		return name;
	}
	
	@Override
	public List<Asset> getAssetList() {
		return assets;
	}

	@Override
	public String getSummary() {
		return summary;
	}
	
	@Override
	public List<String> getCategories() {
		return categories;
	}
	
	/**
	 * Gets all Scenarios from this Recommendation.
	 * @return a List of Scenarios.
	 */
	public List<Scenario> getScenarios() {
		return scenarios;
	}
	
	public String toString() {
		return name;
	}

	
}
