/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package recommendation;

import java.util.ArrayList;
import java.util.List;

import recommendation.asset.Asset;

import attribute.Attribute;

/**
 * This class represents a Scenario of a Recommendation
 * Scenarios contain the information that Attributes contain,
 * a weight, requirement information, and a separate list
 * of Assets.
 * @author Wesley Ripley, Adam Talbot
 * @version 12/4/12
 */
public class Scenario {
	
	private final List<Attribute> attributes;
	private final int weight;
	private final boolean required;
	private final boolean stop;
	private final boolean notRecommended;
	private final List<Asset> assets;
	
	/**
	 * Constructor for a Scenario
	 * @param attributes a List of Attributes for this Scenario
	 * @param weight an integer Weight for this Scenario
	 * @param required true if this Scenario is required to recommend the
	 * Recommendation it is a part of
	 * @param stop true if Scenario processing stops after meeting this Scenario
	 * @param notRecommended true if, by satisfying this Scenario, we DO NOT want
	 * to give this Recommendation.
	 * @param assets a List of Scenario-specific Assets
	 */
	public Scenario(List<Attribute> attributes, int weight, boolean required,
			boolean stop, boolean notRecommended, List<Asset> assets) {
		this.attributes = attributes;
		this.weight = weight;
		this.required = required;
		this.stop = stop;
		this.notRecommended = notRecommended;
		this.assets = assets;
	}
	
	/**
	 * Constructor for a Scenario, with no Assets
	 * @param attributes a List of Attributes for this Scenario
	 * @param weight an integer Weight for this Scenario
	 * @param required true if this Scenario is required to recommend the
	 * Recommendation it is a part of
	 * @param stop true if Scenario processing stops after meeting this Scenario
	 * @param notRecommended true if, by satisfying this Scenario, we DO NOT want
	 * to give this Recommendation.
	 */
	public Scenario(List<Attribute> attributes, int weight, boolean required,
			boolean stop, boolean notRecommended) {
		this(attributes, weight, required, stop, notRecommended, null);
	}
	
	/**
	 * Constructor for a Scenario, with no Assets.  The "stop" and "not recommended"
	 * conditions are false.
	 * @param attributes a List of Attributes for this Scenario
	 * @param weight an integer Weight for this Scenario
	 * @param required true if this Scenario is required to recommend the
	 * Recommendation it is a part of
	 */
	public Scenario(List<Attribute> attributes, int weight, boolean required)
	{
		this(attributes,weight,required,false,false);
	}
	
	/**
	 * Constructor for a Scenario, with no Assets.  The "stop" and "not recommended"
	 * conditions are false.
	 * @param attribInfo for the Name and Value of this Scenario
	 * @param weight integer representing a weight
	 * @param required true if this Scenario must be met to provide the Recommendation
	 */
	public Scenario(Attribute attribInfo, int weight, boolean required) {
		this(new ArrayList<Attribute>(),weight,required);
		attributes.add(attribInfo);
	}

	/**
	 * Simplest constructor for a Scenario.  Provides a weight of "1".
	 * The "required", "stop", and "not recommended" conditions are false. 
	 * @param attribInfo for the Name and Value of this Scenario
	 */
	public Scenario(Attribute attribInfo) {
		this(attribInfo,1,false);
	}
	
	/**
	 * Given a set of constraints determined by the dialog, evaluate
	 * this Scenario.
	 * @param constraints a description of the project given by the dialog
	 * @return a ScenarioResult with the evaluation information
	 */
	public boolean evaluate(Constraints constraints) {
		boolean criteriaMet = true;
		for(Attribute attrNeeded : attributes) {
			Attribute attr = constraints.getAttribute(attrNeeded.getName());
			if(attr == null || !attrNeeded.getValue().fitCriteria(attr.getValue())){
				criteriaMet = false;
				break;
			}
		}
		return criteriaMet;
	}
	
	/**
	 * Gets the Attribute with the Name and Value of this Scenario
	 * @return Attribute containing Name and Value
	 */
	public List<Attribute> getAttributes() {
		return attributes;
	}
	
	/**
	 * Gets the weight of the Scenario
	 * @return int representing Scenario weight
	 */
	public int getWeight()
	{
		return weight;
	}
	
	/**
	 * Determines if this Scenario must be satisfied in order to give the Recommendation
	 * @return true iff this Scenario must be satisfied
	 */
	public boolean isRequired() {
		return required;
	}

	/**
	 * Determines if the processing of the Recommendation should stop after
	 * this Scenario has been evaluated
	 * @return true if the processing should terminate
	 */
	public boolean shouldStop() {
		return stop;
	}

	/**
	 * Determines if, given that this Scenario is satisfied, the
	 * Recommendation must NOT be given.
	 * @return true if the Recommendation should not be given when this
	 * Scenario exists
	 */
	public boolean isNotRecommended() {
		return notRecommended;
	}

	/**
	 * Gets the list of Assets that should be provided with the Recommendation
	 * if this Scenario is satisfied.
	 * @return a List of Assets
	 */
	public List<Asset> getAssets() {
		return assets;
	}
	

}
