/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package recommendation.asset;

import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Builds an Asset based on an identifier and contents.
 * @author Wesley Ripley, Adam Talbot
 * @version 2/20/13
 */
public class AssetBuilder {

	private String identifier;
	private String content;

	/**
	 * Constructor for an AssetBuilder.
	 */
	public AssetBuilder() {
		identifier = null;
		content = null;
	}

	/**
	 * Gets the identifier for this AssetBuilder.
	 * @return a String id, such as "text"
	 */
	public String getIdentifier() {
		return identifier;
	}

	/**
	 * Sets the identifier for this AssetBuilder.
	 * @param id a String id, such as "text"
	 */
	public void setIdentifier(String id) {
		id = id.toLowerCase().trim();
		if (id.endsWith("asset:")) {
			int extraLength = "asset:".length();
			id = id.substring(0, id.length() - extraLength).trim();
		}
		identifier = id;
	}

	/**
	 * Gets the contents of this AssetBuilder
	 * @return a String containing the contents
	 */
	public String getContent() {
		return content;
	}

	/**
	 * Sets the contents of this AssetBuilder
	 * @param content a String containing the contents
	 */
	public void setContent(String content) {
		this.content = content;
	}

	/**
	 * Builds the asset based on the identifier and contents.
	 * @return the newly generated Asset.
	 */
	public Asset buildAsset() {
		if (identifier == null || content == null) {
			throw new IllegalStateException(
					"Cannot make an asset without an identifier and content");
		}
		AssetStore newAsset = new AssetStore(AssetType.fromString(identifier));
		Set<String> tags = newAsset.getContentKeys();
		StringBuilder terminator = new StringBuilder("(");
		for(String tag : tags) {
			terminator.append(tag);
			terminator.append("|");
		}
		terminator.deleteCharAt(terminator.length()-1);
		terminator.append("):");
		Pattern regex = Pattern
				.compile(
						terminator.toString()
								+ "(.*?)"
								+ "("+ terminator.toString() +"|\\z)",
						Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.UNICODE_CASE);
		Matcher matcher = regex.matcher(content);
		int start = 0;
		while (matcher.find(start)) {
			newAsset.addContent(matcher.group(1), matcher.group(2));
			start = matcher.start(3);
		}
		return newAsset;
	}

}
