/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package recommendation.asset;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 * Implements the Asset interface, used for SPC 2.0 Assets.
 * @author Wesley Ripley, Adam Talbot
 * @version 2/20/13
 */
public class AssetStore implements Asset{
	
	private final AssetType type;
	private Map<String,String> contents;
	private final Set<String> contentKeys;
	
	/**
	 * Constructor for an AssetStore, with no contents yet.
	 * @param type the Type of Asset this is going to become
	 */
	public AssetStore(AssetType type) {
		this.type = type;
		contents = new HashMap<String,String>();
		contentKeys = type.getContentKeys();
	}
	
	/**
	 * Constructor for an AssetStore, plus the contents to add.
	 * @param type the Type of Asset this is going to become
	 * @param content a Map of String identifiers to String contents
	 */
	public AssetStore(AssetType type, Map<String,String> content) {
		this(type);
		addContent(content);
	}
	
	/**
	 * Adds contents to this AssetStore.
	 * @param name String identifier for the contents
	 * @param content String representation of the contents
	 * @return true if it was added successfully.
	 */
	public boolean addContent(String name, String content) {
		name = name.trim().toLowerCase();
		if(contentKeys.contains(name)) {
			contents.put(name, content.trim());
			return true;
		}
		return false;
	}
	
	/**
	 * Adds contents to this AssetStore in bulk.
	 * @param content Map of String ids to String contents.
	 * @return true if all contents were added successfully.
	 */
	public boolean addContent(Map<String,String> content) {
		for(String key : content.keySet()) {
			if(!contentKeys.contains(key)) {
				return false;
			}
		}
		for(Entry<String,String> entry : content.entrySet()) {
			contents.put(entry.getKey(), entry.getValue());
		}
		return true;
	}

	@Override
	public Map<String, String> getTemplateTags() {
		return contents;
	}

	@Override
	public Map<String, String> getContents() {
		return contents;
	}

	@Override
	public AssetType getType() {
		return type;
	}
	
	@Override
	public Set<String> getContentKeys() {
		return contentKeys;
	}

}
