/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package recommendationView;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import recommendation.BasicRecommendation;
import recommendation.Scenario;
import recommendation.asset.Asset;
import value.BooleanValue;
import value.IntegerValue;
import value.StringValue;
import value.Value;
import value.valueData.ValueData;
import value.valueData.ValueDataType;
import value.valueData.ValueType;
import attribute.Attribute;

/**
 * This class is used to take a recommendation that would be stored
 * in the database, and reconstruct it into a file that contains a
 * DSL-parseable string.  Useful for being able to edit recommendations.
 * @author Wesley Ripley, Adam Talbot
 * @version 1/20/13
 */
public class RecommendationReconstructor {
	
	private final BasicRecommendation recommendation;
	
	/**
	 * Constructor for a Recommendation Reconstructor.
	 * @param recommendation the Recommendation to be reconstructed
	 */
	public RecommendationReconstructor(BasicRecommendation recommendation){
		this.recommendation = recommendation;
	}
	
	/**
	 * Reconstructs the recommendation that this Reconstructor was
	 * created with.
	 * @return the absolute location of the generated File
	 */
	public String reconstruct(){
		String output = "";
		output = output.concat(
				recommendation() +
				category() +
				summary() +
				description() +
				scenarios() +
				assets(recommendation.getAssetList(), false));
		String fileName = recommendation.getName();
		fileName = fileName.replaceAll("[^a-zA-Z0-9]", "");
		File outputFile = new File(System.getProperty("user.dir") +
				"/recommendations/" + fileName + ".txt");
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(outputFile));
			bw.write(output);
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return outputFile.getAbsolutePath();
	}

	/**
	 * Returns the string for the name of this Recommendation
	 */
	private String recommendation(){
		return "Recommendation: " + recommendation.getName() + "\n";
	}
	
	/**
	 * Returns the string for the Categories of this Recommendation
	 */
	private String category(){
		String output = "";
		for (String c : recommendation.getCategories()){
			output = output.concat("Category: " + c + "\n");
		}
		return output;
	}
	
	/**
	 * Returns the string for the Summary of this Recommendation
	 */
	private String summary(){
		return "Summary: " + recommendation.getSummary() + "\n";
	}
	
	/**
	 * Returns the string for the Description of this Recommendation
	 */
	private String description(){
		return "Description:\n" + recommendation.getDescription() + "\n\n";
	}

	/**
	 * Returns the string that contains all of the Scenarios of
	 * this Recommendation.
	 */
	private String scenarios() {
		String output = "";
		// Run this for each individual scenario
		for(Scenario s : recommendation.getScenarios()){
			output = output.concat(
					"Scenario: \n");
			String given = "";
			// Each individual attribute needs a line of its own
			for(Attribute a : s.getAttributes()){
				// First one says "Given"
				if (given.equals("")){
					given = given.concat("\tGiven ");
					// The rest say "And"
				} else {
					given = given.concat("\tAnd ");
				}
				// Append the name of the Attribute
				given = given.concat(a.getName());
				// Append the condition the Attribute needs to meet
				given = given.concat(valueToString(a.getValue()));
			}
			output = output.concat(given);
			// Move on to all the things that can happen as a result
			// of meeting the conditions
			String then = "";
			int w = s.getWeight();
			if (w != 0){
				then = addThenPrefix(then);
				if (w > 0){
					then = then.concat("Add "+ w + " confidence\n");
				} else {
					then = then.concat("Remove " + (-1*w) + " confidence\n");
				}
			}
			if (s.isRequired()){
				then = addThenPrefix(then);
				then = then.concat("Make Required\n");
			}
			if (s.isNotRecommended()){
				then = addThenPrefix(then);
				then = then.concat("Do Not Recommend\n");
			}
			if (!s.getAssets().isEmpty()){
				then = addThenPrefix(then);
				then = then.concat("Add {\n" +
						assets(s.getAssets(), true) + "\t}\n");
			}
			if (s.shouldStop()){
				then = addThenPrefix(then);
				then = then.concat("stop\n");
			}
			output = output.concat(then + "\n");
		}
		return output;
	}
	
	/**
	 * Used for the Actions of a Scenario.
	 * @param then the current Actions, in String form
	 * @return the correct prefix for the next Action (Then vs And)
	 */
	private String addThenPrefix(String then) {
		if (then.equals("")){
			return then.concat("\tThen ");
		}
		return then.concat("\tAnd ");
	}

	/**
	 * Generates the string that represents a value in the DSL
	 * @param value Self explanatory
	 * @return string of the Value, plus a newline
	 */
	private String valueToString(Value value) {
		// Boolean values are simple and fast.
		if (value.getType().equals(ValueType.BOOLEAN)){
			BooleanValue bvalue = (BooleanValue) value;
			if (bvalue.getContents()){
				return " is true\n";
			} else {
				return " is false\n";
			}
		}
		String output = "";
		// Construct the string for StringValues
		if (value.getType().equals(ValueType.STRING)){
			StringValue svalue = (StringValue) value;
			for (String s : svalue.getContents()){
				if (output.equals("")){
					output = output.concat(" is " + s);
				} else {
					output = output.concat(" or is " + s);
				}
			}
			for (String s : svalue.getNotContents()){
				if (output.equals("")){
					output = output.concat(" is not " + s);
				} else {
					output = output.concat(" or is not " + s);
				}
			}
			return output + "\n";
		}
		// Construct the string for IntegerValues
		if (value.getType().equals(ValueType.INT)){
			IntegerValue ivalue = (IntegerValue) value;
			for (ValueData v : ivalue.getContents()){
				// Add the "or" if it's not the first one.
				if (output.equals("")){
					output = output.concat(" is ");
				} else {
					output = output.concat(" or is ");
				}
				ValueDataType vd = v.getValueDataType();
				// Depends on the type of value data this is
				if (vd.equals(ValueDataType.SINGLE)){
					output = output.concat("" + (int)v.getFirstValue());
				}
				if (vd.equals(ValueDataType.LESS_THAN)){
					output = output.concat("less than ");
					if (v.getInclusive()){
						output = output.concat("or equal to ");
					}
					output = output.concat("" + (int)v.getFirstValue());
				}
				if (vd.equals(ValueDataType.GREATER_THAN)){
					output = output.concat("greater than ");
					if (v.getInclusive()){
						output = output.concat("or equal to ");
					}
					output = output.concat("" + (int)v.getFirstValue());
				}
				if (vd.equals(ValueDataType.BETWEEN)){
					output = output.concat("between " + (int)v.getFirstValue() +
							" and " + (int)v.getSecondValue());
					if(!v.getInclusive()){
						output = output.concat(" exclusive");
					}
				}
				if (vd.equals(ValueDataType.NOT)){
					output = output.concat("not " + (int)v.getFirstValue());
				}
			}
		}
		return output + "\n";
	}

	/**
	 * Returns the string that contains all of the assets of this Recommendation
	 * @param assetList A List of Assets
	 * @param scenario true if we are generating this for the inside of a Scenario
	 */
	private String assets(List<Asset> assetList, boolean scenario) {
		String output = "";
		for(Asset a : assetList){
			String interior = "";
			for(String key : a.getContentKeys()){
				if (scenario){
					interior = interior.concat("\t");
				}
				interior = interior.concat(
						"\t" + key + ": " +
						a.getContents().get(key) + "\n");
			}
			if (scenario){
				output = output.concat("\t");
			}
			output = output.concat(
					a.getType().toString() + " Asset:\n" + interior);
			if (!scenario){
				output = output.concat("\n");
			}
		}
		return output;
	}

}
