/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package recommendationView;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Queue;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import recommendation.RecommendationResult;
import recommendation.asset.Asset;

/**
 * A class to parse through and generate the display for the
 * Recommendations, based on a given template.  The default
 * templates are assumed if none are given. 
 * @author Wesley Ripley, Adam Talbot
 * @version 10/28/12
 */
public class TemplateParser {
	private File recommendationTemplate;
	private File assetTemplate;
	private String token;

	public TemplateParser(String recLocation, String assetLocation, String token){
		recommendationTemplate = new File(recLocation);
		assetTemplate = new File(assetLocation);
		this.token = token;
	}

	public TemplateParser(){
		recommendationTemplate = new File("templates/RecommendationTemplate.txt");
		assetTemplate = new File("templates/AssetTemplate.txt");
		token = "%%";
	}

	/**
	 * Parses the Recommendation and Asset Templates and generates a file
	 * based on the contents of those Templates and the information collected
	 * during the dialog.
	 * @return The absolute path of the generated file
	 * @throws IOException for now.  Eventually should work with the UI
	 * and say that the file is missing.
	 */
	public File parseTemplate(Queue<RecommendationResult> recQueue){
		List<RecommendationResult> recList = new ArrayList<RecommendationResult>();
		// This may look silly, but apparently if you try to do a for each
		// on a queue, it isn't guaranteed to be in order.
		// So we poll it to make a properly ordered list, and use that.
		while(!recQueue.isEmpty()){
			recList.add(recQueue.poll());
		}
		return parseTemplate(recList);
	}

	/**
	 * Parses the Recommendation and Asset Templates and generates a file
	 * based on the contents of those Templates and the information collected
	 * during the dialog.
	 * @return The absolute path of the generated file
	 * @throws IOException for now.  Eventually should work with the UI
	 * and say that the file is missing.
	 */
	public File parseTemplate(List<RecommendationResult> recList){
		// Set up regex patterns
		TemplateParserPatterns patternGenerator = new TemplateParserPatterns(token);
		Pattern fileFinder = patternGenerator.variableFinder("Filename");
		Pattern assetFinder = patternGenerator.tagReplacer("Asset");
		Matcher matcher;
		// Turn the document into a string for easy editing.  It should be short anyway.
		String representation = fileToString(recommendationTemplate);
		if(representation == null){
			return null;
		}
		// Determine the filename
		matcher = fileFinder.matcher(representation);
		matcher.find();
		// Make the file based on it
		File output = new File(matcher.group(1).trim());
		// Make the string we'll be using as the in-memory file
		String outputContents = "";
		// Find the Header (everything before the Recommendations tag
		// Also add it to the output contents.
		String header = collectHeader(representation);
		outputContents = outputContents.concat(header);
		// Find the Footer (everything after the Recommendations tag)
		String footer = collectFooter(representation);
		// Create the Recommendation string that gets added
		String recDefault = collectInterior("Recommendations", representation);
		// Create the Asset string that gets added
		String assetDefault = collectInterior("Assets", representation);
		// For each Recommendation, append a string.
		for(RecommendationResult r : recList){
			String tempContents = recDefault;
			//Replace Recommendation tags (like Name, Description, etc)
			tempContents = replaceTags(patternGenerator, r, tempContents);
			// For each Asset, append a string
			String assetContents = "";
			for(Asset a : r.getAssetList()){
				String tempSubcontents = assetDefault;
				// Replace Asset
				matcher = assetFinder.matcher(tempSubcontents);
				String tempAssets = parseAsset(a);
				if(tempAssets == null){
					return null;
				}
				tempSubcontents = matcher.replaceFirst(tempAssets);
				// Add to the Asset string
				assetContents = assetContents.concat(tempSubcontents);
			}
			// Replace the Asset portion
			Pattern assetStart = patternGenerator.tagReplacer("Assets");
			Pattern assetEnd = patternGenerator.tagReplacer("End\\sAssets");
			matcher = assetStart.matcher(tempContents);
			matcher.find();
			int begin = matcher.start();
			matcher = assetEnd.matcher(tempContents);
			matcher.find();
			int end = matcher.end();
			String tempFirst = tempContents.substring(0,begin);
			String tempSecond = tempContents.substring(end);

			// Add the final collected string to the output contents
			outputContents = outputContents + tempFirst + assetContents + tempSecond;
		}
		// Add the footer
		outputContents = outputContents.concat(footer);
		// Write the output contents to the file
		try{
			output.getParentFile().mkdirs();
			BufferedWriter bw = new BufferedWriter(new FileWriter(output));
			bw.write(outputContents);
			bw.close();
		}catch(IOException e){
			e.printStackTrace();
			return null;
		}

		// Return the path of the file
		return output;
	}

	/**
	 * Collects everything below the End Recommendations tag.
	 * @param representation the Rec Template as text
	 * @return a String containing what we need
	 */
	private String collectFooter(String representation) {
		TemplateParserPatterns patternGenerator = new TemplateParserPatterns(token);
		Pattern endRecFinder = patternGenerator.tagReplacer("End\\sRecommendations");
		Matcher matcher = endRecFinder.matcher(representation);
		// Find where the footer starts
		matcher.find();
		int offset = matcher.end();
		// Strip away the rest of the String
		String footer = representation.substring(offset);
		return footer;
	}

	/**
	 * Collects everything above the Recommendations tag.
	 * @param representation the Rec Template as text
	 * @return a String containing what we need
	 */
	private String collectHeader(String representation) {
		TemplateParserPatterns patternGenerator = new TemplateParserPatterns(token);
		Pattern fileFinder = patternGenerator.variableFinder("Filename");
		Pattern recFinder = patternGenerator.tagReplacer("Recommendations");
		Matcher matcher = fileFinder.matcher(representation);
		// Find where the header starts
		int offset = 0;
		if (matcher.find()){
			offset = matcher.end();
		}
		// Find where the header ends
		matcher = recFinder.matcher(representation);
		matcher.find();
		int termination = matcher.start();
		// Strip away the non-header part of the string
		String header = representation.substring(offset, termination);
		return header;
	}

	/**
	 * A utility for the Template parser that handles
	 * the parsing of the Asset Template.
	 * @param a the Asset provided
	 * @return the generated asset display
	 * @throws IOException 
	 */
	private String parseAsset(Asset a){
		// Setup
		String representation = fileToString(assetTemplate);
		if (representation == null){
			return null;
		}
		TemplateParserPatterns patternGenerator = new TemplateParserPatterns(token);
		// Find the asset template we are looking for
		String outputContents = collectInterior(a.getType() + "\\sAsset", representation);
		// For each tag, replace it
		outputContents = replaceTags(patternGenerator,a,outputContents);
		// Output contents should now be exactly what we need. 
		return outputContents;
	}

	private String fileToString(File template){
		// Set up file reader
		try{
			BufferedReader br = new BufferedReader(new FileReader(template));
			String representation = "";
			while(br.ready()){
				representation = representation.concat(br.readLine() + "\n");
			}
			br.close();
			return representation;
		}catch(IOException e){
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Collects the interior of a loop
	 * @param tag The loop name
	 * @param representation The full string
	 * @return The string inside the loop
	 */
	private String collectInterior(String tag, String representation){
		String output = "";
		TemplateParserPatterns patternGenerator = new TemplateParserPatterns(token);
		// Create a pattern for both the start and end
		Pattern xBound = patternGenerator.tagReplacer("(?:End\\s)?" + tag);
		Matcher matcher;
		matcher = xBound.matcher(representation);
		// Find the start, log the position
		matcher.find();
		int start = matcher.end();
		// Find the end, log the position
		matcher.find();
		int end = matcher.start();
		// Create a pattern for any character

		matcher = Pattern.compile("(.|\\n)").matcher(representation);
		matcher = matcher.region(start, end);
		// Append to the string until we've collected the whole thing
		while(true){
			matcher.find();
			if (matcher.hitEnd())
				break;
			output = output.concat(matcher.group(1));
		}
		return output;
	}
	
	private String replaceTags(TemplateParserPatterns patternGenerator,Templateable templateable, String template) {
		
		Matcher matcher;
		for(Entry<String, String> entry : templateable.getTemplateTags().entrySet())
		{
			Pattern tagFinder = patternGenerator.tagReplacer(entry.getKey());
			matcher = tagFinder.matcher(template);
			template = matcher.replaceFirst(entry.getValue());
		}
		return template;
	}
	

}
