/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package value;

import java.util.ArrayList;
import java.util.List;

import value.valueData.ValueData;
import value.valueData.ValueDataType;
import value.valueData.ValueType;

/**
 * An implementation of the Value class that represents an Integer.
 * @author Wesley Ripley, Adam Talbot
 * @version 2/20/13
 */
public class IntegerValue implements Value {
	
	private List<ValueData> contents;

	/**
	 * Constructor for a IntegerValue, with no data yet
	 */
	public IntegerValue(){
		this.contents = new ArrayList<ValueData>();
	}
	
	/**
	 * Constructor for a IntegerValue starting with a SINGLE value
	 * @param singleValue the single value this IntegerValue contains
	 */
	public IntegerValue(int singleValue){
		this.contents = new ArrayList<ValueData>();
		contents.add(new ValueData(ValueDataType.SINGLE, singleValue));
	}
	
	/**
	 * Constructor for an IntegerValue with the default BETWEEN value.
	 * @param start the starting endpoint.
	 * @param end the finishing endpoint.
	 */
	public IntegerValue(int start, int end){
		this.contents = new ArrayList<ValueData>();
		this.addBetween(start, end);
	}

	/**
	 * Adds exactly one int to this IntegerValue
	 * @param singleValue The int to add.
	 */
	public void addSingle(int singleValue){
		this.contents.add(new ValueData(ValueDataType.SINGLE, singleValue));
	}
	
	/**
	 * Adds all integers less than the given, not inclusive.
	 * @param lessThan An integer endpoint
	 */
	public void addLessThan(int lessThan){
		this.contents.add(new ValueData(ValueDataType.LESS_THAN, lessThan));
	}
	
	/**
	 * Adds all integers less than the given
	 * @param lessThan An integer endpoint
	 * @param inclusive true if we should also include the endpoint
	 */
	public void addLessThan(int lessThan, boolean inclusive){
		this.contents.add(new ValueData(ValueDataType.LESS_THAN, lessThan, inclusive));
	}
	
	/**
	 * Adds all integers greater than the given, not inclusive
	 * @param greaterThan An integer endpoint
	 */
	public void addGreaterThan(int greaterThan){
		this.contents.add(new ValueData(ValueDataType.GREATER_THAN, greaterThan));
	}
	
	/**
	 * Adds all integers greater than the given
	 * @param greaterThan An integer endpoint
	 * @param inclusive true if we should also include the endpoint
	 */
	public void addGreaterThan(int greaterThan, boolean inclusive){
		this.contents.add(new ValueData(ValueDataType.GREATER_THAN, greaterThan, inclusive));
	}
	
	/**
	 * Adds all integers between and including the given endpoints 
	 * @param start An integer starting endpoint
	 * @param end An integer finishing endpoint
	 */
	public void addBetween(int start, int end){
		this.contents.add(new ValueData(ValueDataType.BETWEEN, start, end));
	}
	
	/**
	 * Adds all integers between the given endpoints
	 * @param start An integer starting endpoint
	 * @param end An integer finishing endpoint
	 * @param inclusive true if we should also include these endpoints
	 */
	public void addBetween(int start, int end, boolean inclusive){
		this.contents.add(new ValueData(ValueDataType.BETWEEN, start, end, inclusive));
	}
	
	/**
	 * Adds every integer except for the given integer
	 * @param not The integer to exclude
	 */
	public void addNot(int not){
		this.contents.add(new ValueData(ValueDataType.NOT, not));
	}
	
	@Override
	public void initialize(){
		this.contents.clear();
	}

	@Override
	public boolean isMultipleValue() {
		// False if it contains nothing
		if (this.contents.size() == 0){
			return false;
		}
		// False if it contains exactly one object in the contents,
		// and it is a SINGLE value.
		if (this.contents.size() == 1 &&
				this.contents.get(0).getValueDataType().equals(ValueDataType.SINGLE)){
			return false;
		}
		// Otherwise, true.
		return true;
	}

	@Override
	public boolean fitCriteria(Value val) {
		// If it isn't even an INT, just return false.
		if (!val.getType().equals(ValueType.INT)){
			return false;
		}
		// We now know it must be an IntegerValue.
		// Cast it as one so we can do important things with it.
		IntegerValue intVal = (IntegerValue) val;
		List<ValueData> compareContents = new ArrayList<ValueData>();
		compareContents.addAll(intVal.getContents());
		// We now can compare "this.contents" to "compareContents".
		// If we ever find a match, just return true and stop there.
		// Otherwise, continue until we can confirm false.
		// TODO: Implement an "All" variant.  This is an "Any" variant.
		// I am aware this can be done in much less code.  It is written
		// in this way to ensure that it's easy to understand.
		for (ValueData t : this.contents){
			for (ValueData c : compareContents){
				switch(t.getValueDataType()){
				case SINGLE:
					switch(c.getValueDataType()){
					case SINGLE: // This: Single, Compare: Single
						if (t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case LESS_THAN: // This: Single, Compare: Less Than
						if (t.getFirstValue() < c.getFirstValue()){
							return true;
						}
						if (c.getInclusive() && t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case GREATER_THAN: // This: Single, Compare: Greater Than
						if (t.getFirstValue() > c.getFirstValue()){
							return true;
						}
						if (c.getInclusive() &&
								t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case BETWEEN: // This: Single, Compare: Between
						if (t.getFirstValue() > c.getFirstValue() &&
								t.getFirstValue() < c.getSecondValue()){
							return true;
						}
						if (c.getInclusive() &&
								(t.getFirstValue() == c.getFirstValue() ||
								t.getFirstValue() == c.getSecondValue())){
							return true;
						}
						break;
					case NOT: // This: Single, Compare: Not
						// Note: Think of "Not" as an "Everything Except."
						// Everything is an OR operation.
						if (t.getFirstValue() != c.getFirstValue()){
							return true;
						}
					}
					break;
				case LESS_THAN:
					switch(c.getValueDataType()){
					case SINGLE: // This: Less Than, Compare: Single
						if (t.getFirstValue() > c.getFirstValue()){
							return true;
						}
						if (t.getInclusive() &&
								t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case LESS_THAN: // This: Less Than, Compare: Less Than
						return true; // This will ALWAYS be true.
					case GREATER_THAN: // This: Less Than, Compare: Greater Than
						if (t.getFirstValue() > c.getFirstValue()){
							return true;
						}
						if (t.getInclusive() && c.getInclusive() &&
								t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case BETWEEN: // This: Less Than, Compare: Between
						if (t.getFirstValue() > c.getFirstValue()){
							return true;
						}
						if (t.getInclusive() && c.getInclusive() &&
								t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case NOT: // This: Less Than, Compare: Not
						return true; // This will ALWAYS be true.
					}
					break;
				case GREATER_THAN:
					switch(c.getValueDataType()){
					case SINGLE: // This: Greater Than, Compare: Single
						if (t.getFirstValue() < c.getFirstValue()){
							return true;
						}
						if (t.getInclusive() &&
								t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case LESS_THAN: // This: Greater Than, Compare: Less Than
						if (t.getFirstValue() < c.getFirstValue()){
							return true;
						}
						if (t.getInclusive() && c.getInclusive() &&
								t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case GREATER_THAN: // This: Greater Than, Compare: Greater Than
						return true; // This is ALWAYS true.
					case BETWEEN: // This: Greater Than, Compare: Between
						if (t.getFirstValue() < c.getSecondValue()){
							return true;
						}
						if (t.getInclusive() && c.getInclusive() &&
								t.getFirstValue() == c.getSecondValue()){
							return true;
						}
						break;
					case NOT: // This: Greater Than, Compare: Not
						return true; // ALWAYS true.
					}
					break;
				case BETWEEN:
					switch(c.getValueDataType()){
					case SINGLE: // This: Between, Compare: Single
						if (t.getFirstValue() < c.getFirstValue() &&
								t.getSecondValue() > c.getFirstValue()){
							return true;
						}
						if (t.getInclusive() &&
								(t.getFirstValue() == c.getFirstValue() ||
								t.getSecondValue() == c.getFirstValue())){
							return true;
						}
						break;
					case LESS_THAN: // This: Between, Compare: Less Than
						if (t.getFirstValue() < c.getFirstValue()){
							return true;
						}
						if (t.getInclusive() && c.getInclusive() &&
								t.getFirstValue() == c.getFirstValue()){
							return true;
						}
						break;
					case GREATER_THAN: // This: Between, Compare: Greater Than
						if (t.getSecondValue() > c.getFirstValue()){
							return true;
						}
						if (t.getInclusive() && c.getInclusive() &&
								t.getSecondValue() == c.getFirstValue()){
							return true;
						}
						break;
					case BETWEEN: // This: Between, Compare: Between
						if (t.getFirstValue() > c.getFirstValue() &&
								t.getFirstValue() < c.getSecondValue()){
							return true;
						}
						if (t.getSecondValue() < c.getSecondValue() &&
								t.getSecondValue() > c.getFirstValue()){
							return true;
						}
						if (t.getFirstValue() < c.getFirstValue() &&
								t.getSecondValue() > c.getSecondValue()){
							return true;
						}
						if (t.getInclusive() && c.getInclusive() &&
								(t.getSecondValue() == c.getFirstValue() ||
								t.getFirstValue() == c.getSecondValue())){
							return true;
						}
						break;
					case NOT: // This: Between, Compare: Not
						return true; // Always true again.
					}
					break;
				case NOT:
					switch(c.getValueDataType()){
					case SINGLE: // This: Not, Compare: Single
						if (t.getFirstValue() != c.getFirstValue()){
							return true;
						}
						break;
					case LESS_THAN: // This: Not, Compare: Less Than
						return true;
					case GREATER_THAN: // This: Not, Compare: Greater Than
						return true;
					case BETWEEN: // This: Not, Compare: Between
						return true;
					case NOT: // This: Not, Compare: Not
						return true;
					}
				}
			} // End (ValueData c : compareContents)
		} // End (ValueData t : this.contents)
		
		// We found no matches.  Return false.
		return false;
	}

	@Override
	public ValueType getType() {
		return ValueType.INT;
	}
	
	/**
	 * Gets the contents list for this Integer Value
	 * @return a List of ValueData
	 */
	public List<ValueData> getContents(){
		return this.contents;
	}
	
	public boolean equals(Object o) {
		if(!(o instanceof IntegerValue)) {
			return false;
		}
		IntegerValue other = (IntegerValue)o;
		return this.contents.equals(other.getContents());
	}

	@Override
	public boolean isEmpty() {
		return contents.isEmpty();
	}

}
