/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package value;

import java.util.ArrayList;
import java.util.List;

import value.valueData.ValueType;

/**
 * An implementation of the Value class that represents a String.
 * @author Wesley Ripley, Adam Talbot
 * @version 10/03/12
 */
public class StringValue implements Value {

	private List<String> contents;
	private List<String> notContents;
	
	/**
	 * Constructor for a StringValue, starting with a single String
	 * @param value as a String
	 */
	public StringValue(String value) {
		this.contents = new ArrayList<String>();
		this.notContents = new ArrayList<String>();
		contents.add(value);
	}
	
	/**
	 * Constructor for a StringValue, with no data yet
	 */
	public StringValue(){
		this.contents = new ArrayList<String>();
		this.notContents = new ArrayList<String>();
	}
	
	/**
	 * Adds a String to this StringValue
	 * @param value the String to be added
	 */
	public void addString(String value){
		this.contents.add(value);
	}
	
	/**
	 * Adds a "not equal to" String to this StringValue
	 * Note: Multiple unique values here will result in all Strings
	 * matching this Value.
	 * @param value the String to be excluded
	 */
	public void addNotString(String value){
		this.notContents.add(value);
	}
	
	@Override
	public void initialize(){
		this.contents.clear();
	}
	
	/**
	 * Gets the Strings that this Value represents.  Ignores "not"s.
	 * @return List of Strings that represents this StringValue
	 */
	public List<String> getContents() {
		return contents;
	}
	
	/**
	 * Gets the Strings that this Value represents in the form of "not"s.
	 * @return List of (usually 0 or 1) Strings that define what
	 * this StringValue cannot be.  More than 1 usually means everything
	 * is true, because they are "or"s.
	 */
	public List<String> getNotContents() {
		return notContents;
	}

	@Override
	public boolean fitCriteria(Value val) {
		// If it isn't even a STRING, just return false.
		if (!val.getType().equals(ValueType.STRING)){
			return false;
		}
		// This must be a StringValue, then.
		StringValue strVal = (StringValue) val;
		List<String> compareContents = new ArrayList<String>();
		compareContents.addAll(strVal.getContents());
		// Compare each string to each other string and find matches.
		// Note: This version is NOT CASE SENSITIVE.
		for (String t : this.contents){
			for (String c : compareContents){
				if (t.equalsIgnoreCase(c)){
					return true;
				}
			}
		}
		// Compare each not string to each given string
		// and find matches.
		// Note: This version is NOT CASE SENSITIVE.
		for (String t : this.notContents){
			for (String c : compareContents){
				if(!t.equalsIgnoreCase(c)){
					return true;
				}
			}
		}
		// We didn't find a match.
		return false;
	}

	@Override
	public boolean isMultipleValue() {
		if (contents.size() <= 1 && notContents.isEmpty()){
			return false;
		}
		return true;
	}

	@Override
	public ValueType getType() {
		return ValueType.STRING;
	}
	
	public boolean equals(Object o) {
		if(!(o instanceof StringValue)) {
			return false;
		}
		StringValue other = (StringValue)o;
		boolean e = (contents.equals(other.getContents()) &&
				notContents.equals(other.getNotContents()));
		return e;
	}

	@Override
	public boolean isEmpty() {
		boolean e = contents.isEmpty() && notContents.isEmpty();
		return e;
	}

}
