/*******************************************************************************
 * This file is part of the Software Process Configurator MQP
 * Project Members: Wesley Ripley, Adam Talbot.
 * Project Advisor: Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wesley Ripley, Adam Talbot
 *******************************************************************************/

package value.valueData;

/**
 * Value Data for any number.  Construction of a ValueData ensures that the given
 * data is valid.
 * @author Wesley Ripley, Adam Talbot
 * @version 1/16/13
 */
public final class ValueData {
	private ValueDataType type;
	private double firstValue;
	private double secondValue;
	private boolean inclusive;
	
	/**
	 * Constructor for SINGLE and NOT ValueData, or for LESS_THAN or
	 * GREATER_THAN with the default of not being inclusive.
	 * @param type A relevant ValueDataType
	 * @param firstValue The only relevant number required
	 */
	public ValueData(ValueDataType type, double firstValue){
		if (type.equals(ValueDataType.SINGLE) ||
				type.equals(ValueDataType.GREATER_THAN) ||
				type.equals(ValueDataType.LESS_THAN) ||
				type.equals(ValueDataType.NOT)){
			// Set data if we have a relevant Data Type
			this.type = type;
			this.firstValue = firstValue;
			// Inclusive defaults to false for LESS_THAN, GREATER_THAN, and NOT.
			if (type.equals(ValueDataType.SINGLE)){
				this.inclusive = true;
			} else {
				this.inclusive = false;
			}
		} else {
			throw new IllegalStateException("Invalid ValueData given");
		}
	}
	
	/**
	 * Constructor for LESS_THAN and GREATER_THAN ValueData
	 * @param type A relevant ValueDataType
	 * @param firstValue The only relevant number required
	 * @param inclusive true if the range also includes firstValue
	 */
	public ValueData(ValueDataType type, double firstValue, boolean inclusive){
		if (type.equals(ValueDataType.GREATER_THAN) ||
				type.equals(ValueDataType.LESS_THAN)){
			this.type = type;
			this.firstValue = firstValue;
			this.inclusive = inclusive;
		} else {
			throw new IllegalStateException("Invalid ValueData given");
		}
	}
	
	/**
	 * Constructor for BETWEEN ValueData with the default of being inclusive
	 * @param type A relevant ValueDataType
	 * @param firstValue The start of the range
	 * @param secondValue The end of the range
	 */
	public ValueData(ValueDataType type, double firstValue, double secondValue){
		if (type.equals(ValueDataType.BETWEEN) && firstValue < secondValue){
			this.type = type;
			this.firstValue = firstValue;
			this.secondValue = secondValue;
			this.inclusive = true;
		} else {
			throw new IllegalStateException("Invalid ValueData given");
		}
	}
	
	/**
	 * Constructor for BETWEEN ValueData
	 * @param type A relevant ValueDataType
	 * @param firstValue The start of the range
	 * @param secondValue The end of the range
	 * @param inclusive true if the range includes the end points
	 */
	public ValueData(ValueDataType type, double firstValue, double secondValue, boolean inclusive){
		if (type.equals(ValueDataType.BETWEEN) && firstValue < secondValue){
			this.type = type;
			this.firstValue = firstValue;
			this.secondValue = secondValue;
			this.inclusive = inclusive;
		} else {
			throw new IllegalStateException("Invalid ValueData given");
		}
	}
	
	/**
	 * Gets the type of data this contains
	 * @return a ValueDataType
	 */
	public ValueDataType getValueDataType(){
		return type;
	}
	
	/**
	 * Gets the first number, usually the only relevant number in the Value
	 * @return a double containing the number
	 */
	public double getFirstValue(){
		return firstValue;
	}
	
	/**
	 * Gets the inclusivity of this ValueData
	 * @return true if inclusive, false if not inclusive
	 */
	public boolean getInclusive(){
		return inclusive;
	}

	/**
	 * Gets the second number, which only exists for BETWEEN types.
	 * @return a double containing the number
	 */
	public double getSecondValue(){
		return secondValue;
	}
	
	public boolean equals(Object o) {
		if(!(o instanceof ValueData)) {
			return false;
		}
		ValueData other = (ValueData)o;
		boolean equal = this.type == other.getValueDataType() && 
						this.firstValue == other.getFirstValue() && 
						this.inclusive == other.getInclusive();
		if(type == ValueDataType.BETWEEN) {
			equal = equal && this.secondValue == other.getSecondValue();
		}
		return equal;
	}
	
}
