/*******************************************************************************
 * This file is part of Jake Todaro and Pragathi Balasubramanian's MQP.
 * Software Process Configurator MQP
 * Advised by Professor Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jake Todaro
 *******************************************************************************/

package xmlParser;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import value.IntegerValue;
import value.StringValue;
import value.Value;
import value.valueData.ValueType;
import attribute.AttributeDescription;
import attribute.AttributeReference;
import booleanExpression.AndExpression;
import booleanExpression.BooleanExpression;
import booleanExpression.BooleanValue;
import booleanExpression.NotExpression;
import booleanExpression.OrExpression;

import dialog.Answer;
import dialog.Dialog;
import dialog.Question;
import dialog.SubDialog;
import dialog.UniqueQuestionIdException;

/**
 * this class handles all of the parsing of a sub dialog. the xml file is parsed
 * and new SubDialog, Question, Answer, and Recommendation objects are created.
 * see the xmlDialogTutorial to learn how to write xml SubDialogs for this
 * parser.
 * 
 * @author Jake Todaro
 * 
 * @version 12/09/11
 */
public class SubDialogParser {

	private static SubDialog subDialog = null;

	/**
	 * parse the XML file with the specified name and return a SubDialog
	 * corresponding to the XML subdialog
	 * 
	 * @param fileName
	 *            - the name of the XML subdialog file
	 * @return a SubDialog corresponding to the XML subdialog
	 */
	public static SubDialog parseSubDialog(String fileName) {

		final Element subDialogElement = XMLParser.getDocumentElement(fileName);

		// get the title of the SubDialog and create the SubDialog
		subDialog = new SubDialog(subDialogElement.getAttribute("title"));

		// look for SubDialogTriggers
		final NodeList childList = subDialogElement.getChildNodes();
		for (int i = 0; i < childList.getLength(); i++) {
			if (childList.item(i).getNodeName().equals("Trigger")) {
				subDialog.setTrigger(parseTrigger((Element) childList.item(i)));
				break;
			}
		}

		// get a nodelist of question elements
		final NodeList questionNodeList = subDialogElement
				.getElementsByTagName("Question");

		// if there are questions
		if (questionNodeList != null && questionNodeList.getLength() > 0) {
			for (int i = 0; i < questionNodeList.getLength(); i++) {

				// get the question element
				Element questionElement = (Element) questionNodeList.item(i);

				// create the question object from the question element
				Question question = parseQuestion(questionElement);

				// add question to the SubDialog
				subDialog.addQuestion(question);
			}
		}

		return subDialog;
	}

	/**
	 * creates a Question object represented by the question element
	 * 
	 * @param questionElement
	 *            - the element that represents the question
	 * @param dialog
	 *            - the dialog that this Question and SubDialog belong to
	 * @return the Question object represented by the question element
	 */
	private static Question parseQuestion(Element questionElement) {

		final String questionStatement = questionElement
				.getAttribute("questionStatement");
		final String attribute = questionElement.getAttribute("attribute");
		AttributeReference attributeRef = Dialog.getAttributeReference();
		AttributeDescription attributeDesc = attributeRef.getAttr(attribute);
		final Question question = new Question(questionStatement,attributeDesc);

		String id = (questionElement.getAttribute("id"));
		//if no id is given, give it the default id
		if(id.equals("")){
			id = Integer.toString(subDialog.getQuestionList().size());
		}
		//check id is unique
		if(subDialog.getQuestionById(id) != null){
			
			try {
				throw new UniqueQuestionIdException("This question id is not unique to this SubDialog.");
			} catch (UniqueQuestionIdException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		question.setID(id);

		parseAndAddAnswers(questionElement, question);

		final NodeList triggerNodeList = questionElement
				.getElementsByTagName("Trigger");
		if (triggerNodeList.getLength() > 0) {
			final Element triggerElement = (Element) triggerNodeList.item(0);
			question.setTrigger(parseTrigger(triggerElement));
		}

		return question;
	}

	/**
	 * Parse and adds Answer objects associated with a particular Question
	 * 
	 * @param questionElement
	 *            - the xml element representing a question
	 * @param question
	 *            - the Question object that corresponds to the xml element
	 */
	private static void parseAndAddAnswers(Element questionElement,
			Question question) {

		final NodeList answerNodeList = questionElement
				.getElementsByTagName("Answer");
		if (answerNodeList != null && answerNodeList.getLength() > 0) {
			for (int i = 0; i < answerNodeList.getLength(); i++) {

				Element answerElement = (Element) answerNodeList.item(i);
				String answerText = answerElement.getAttribute("text");
				Value value = parseValue(answerElement,question);
				Answer answer = new Answer(i + 1, answerText,value);


				question.addAnswer(answer);
			}
		}
	}
	
	private static Value parseValue(Element answerElement, Question question) {
		Value value;
		AttributeDescription attrDesc = question.getAttributeDescription();
		if(attrDesc == null) {
			return null;
		}
		if(answerElement.hasAttribute("value")) {
			String valueString = answerElement.getAttribute("value");
			
			if(valueString != null && !valueString.equals("")) {
				value = attrDesc.getType().makeValue(valueString);
			} else {
				//TODO: throw exception?
				value = null;
			}
		} else {
			final NodeList valueNodeList = answerElement.getElementsByTagName("Value");
			if(valueNodeList != null && valueNodeList.getLength() > 0) {
				
				ValueType valueType = attrDesc.getType();
				switch(valueType) {
				case STRING:
					value = new StringValue();
					break;
				case INT:
					value = new IntegerValue();
					break;
				case BOOLEAN:
					throw new IllegalStateException("Cannot have multiple boolean values. Must be specified with answer");
				default:
					throw new IllegalArgumentException("Illegal value type");
				}
				for(int i = 0; i < valueNodeList.getLength(); i++) {
					Element valueElement = (Element) valueNodeList.item(i);
					
					String operator = valueElement.getAttribute("type").toLowerCase();
					switch(valueType) {
					case STRING:
						if(!operator.equals("equal")) {
							//TODO: Throw exception or not? Which exception?
							throw new IllegalStateException("Invalid operator for a non integer type");
						}
						((StringValue) value).addString(valueElement.getAttribute("value"));
						break;
					case INT:
						IntegerValue intValue = (IntegerValue)value;
						if(operator.equals("between")) {
							String startString = valueElement.getAttribute("start");
							String endString = valueElement.getAttribute("end");
							if(startString == null || startString.equals("") || endString == null || endString.equals("")) {
								throw new IllegalStateException("Between must have both start and end");
							}
							try{
								int start = Integer.parseInt(startString);
								int end = Integer.parseInt(endString);
								boolean inclusive = getInclusive(valueElement,true);
								intValue.addBetween(start, end,inclusive);
							} catch (NumberFormatException e) {
								throw new IllegalArgumentException("Integer values must be an integer");
							}
						} else {
							String valueString = valueElement.getAttribute("value");
							if(valueString == null || valueString.equals("")) {
								throw new IllegalStateException("Must provide a value");
							}
							try {
								int valueNum = Integer.parseInt(valueString);
								if(operator.equals("equal")) {
									intValue.addSingle(valueNum);
								} else if(operator.equals("not")) {
									intValue.addNot(valueNum);
								} else {
									boolean inclusive = getInclusive(valueElement,false);
									if(operator.equals("lessthan")) {
										intValue.addLessThan(valueNum, inclusive);
									} else if(operator.equals("greaterthan")) {
										intValue.addGreaterThan(valueNum, inclusive);
									}
								}
							} catch (NumberFormatException e) {
								throw new IllegalArgumentException("Integer values must be an integer");
							}
						}
						break;
					default:
						throw new IllegalArgumentException("Illegal value type");
					}
				}
				if(value.isEmpty()) {
					value = null;
				}
			} else {
				//TODO: throw exception?
				value = null;
			}
		}
		return value;
	}
	
	public static boolean getInclusive(Element element, boolean def) {
		if(element.hasAttribute("inclusive")) {
			return Boolean.parseBoolean(element.getAttribute("inclusive"));
		} else {
			return def;
		}
	}


	/**
	 * parse and return the trigger element
	 * 
	 * @param triggerElement
	 *            - the Element to parse
	 * @return the BooleanValue representing this Trigger
	 */
	public static BooleanValue parseTrigger(Element triggerElement) {
		final NodeList booleanValueList = triggerElement.getChildNodes();
		return parseBooleanValue((Element) booleanValueList.item(1));
	}

	/**
	 * parse the specified Element into a BooleanValue and return it
	 * 
	 * @param booleanValueElement
	 *            - the Element to parse
	 * @return a BooleanValue corresponding to the Element
	 */
	public static BooleanValue parseBooleanValue(Element booleanValueElement) {

		if (booleanValueElement.getNodeName().equals("and")
				|| booleanValueElement.getNodeName().equals("or")) {

			BooleanExpression booleanExpression = null;
			if (booleanValueElement.getNodeName().equals("and")) {
				booleanExpression = new AndExpression();
			}
			if (booleanValueElement.getNodeName().equals("or")) {
				booleanExpression = new OrExpression();
			}

			final NodeList childrenElements = booleanValueElement
					.getChildNodes();

			for (int i = 0; i < childrenElements.getLength(); i++) {
				if (!childrenElements.item(i).getNodeName().equals("#text")) {
					booleanExpression
							.addAtom(parseBooleanValue((Element) childrenElements
									.item(i)));
				}
			}

			return booleanExpression;
		}

		if (booleanValueElement.getNodeName().equals("not")) {
			final Element subElement = (Element) booleanValueElement
					.getChildNodes().item(1);
			return new NotExpression(parseBooleanValue(subElement));

		}

		if (booleanValueElement.getNodeName().equals("question")) {
			return parseAnswerBooleanValue(booleanValueElement);

		}

		return null;
	}

	/**
	 * parse the specified Element into an Answer and return it
	 * 
	 * @param booleanValueElement
	 *            - the Element to parse
	 * @return an Answer corresponding to the Element
	 */
	public static Answer parseAnswerBooleanValue(Element booleanValueElement) {
		final String questionID = booleanValueElement
				.getAttribute("question");
		final int answerNum = Integer.parseInt(booleanValueElement
				.getAttribute("answer"));

		// check if the trigger references another SubDialog
		final String references = booleanValueElement
				.getAttribute("references");
		
		SubDialog referenceSubDialog = subDialog;
		if (!references.equals("")) {
			if(Dialog.findSubDialogByTitle(references) == null){
				try {
					throw new SubDialogNotFoundException("SubDialog " + references + " not found");
				} catch (SubDialogNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			else{
				referenceSubDialog = Dialog.findSubDialogByTitle(references);
			}
		}
		
		//check that questionIS is valid
		if(referenceSubDialog.getQuestionById(questionID) == null){
			try {
				throw new QuestionIdNotFoundException("Question id " + questionID + " not found");
			} catch (QuestionIdNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		return referenceSubDialog.getQuestionById(questionID).getAnswerList().get(answerNum - 1);
	}


}
