/*******************************************************************************
 * This file is part of Jake Todaro and Pragathi Balasubramanian's MQP.
 * Software Process Configurator MQP
 * Advised by Professor Gary Pollice
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jake Todaro
 *******************************************************************************/

package dialog;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.PriorityQueue;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import recommendation.BasicRecommendation;
import recommendation.Recommendation;
import recommendation.RecommendationResult;
import recommendation.Scenario;
import recommendation.asset.Asset;
import recommendation.asset.AssetStore;
import recommendation.asset.AssetType;
import value.BooleanValue;
import value.StringValue;
import value.valueData.ValueType;
import xmlParser.DialogParser;
import attribute.Attribute;
import attribute.AttributeDescription;
import attribute.AttributeReference;
import database.Database;
import database.TestDatabase;

public class TestRunningDialog {

	private static SubDialog sd1, sd2;
	
	@BeforeClass
	public static void setup(){
		
		DialogParser.parseDialog("xmlDialogs/[DIALOG] testIndex.xml",dialogDatabase());
	
		sd1 = Dialog.getSubDialogList().get(0);
		sd2 = Dialog.getSubDialogList().get(1);	
	}
	
	@AfterClass
	public static void tearDown(){
		Dialog.close();
	}
	
	@Test
	public void testGetFirstQuestion() {
		assertEquals(sd1.getQuestionList().get(0),Dialog.getNextQuestion());		
	}
	
	@Test
	public void testGetNextTriggeredQuestion() {
		sd1.getQuestionList().get(0).getAnswerList().get(1).select();
		assertEquals(sd1.getQuestionList().get(1),Dialog.getNextQuestion());
		assertEquals(new StringValue("thirsty"),Dialog.getAttribute("hungryOrThirsty").getValue());
	}
	
	/** run through questions to get to next testable question*/
	@Test
	public void testRunning3_6() {
		sd1.getQuestionList().get(1).getAnswerList().get(0).select();
		
		assertTrue(Dialog.getNextQuestion().equals(sd1.getQuestionList().get(2)));
		assertEquals(new BooleanValue(true),Dialog.getAttribute("21+").getValue());
		
		sd1.getQuestionList().get(2).getAnswerList().get(3).select();
		assertTrue(Dialog.getNextQuestion().equals(sd1.getQuestionList().get(3)));
		assertEquals(new StringValue("formal dinner"),Dialog.getAttribute("mealType").getValue());
		
		sd1.getQuestionList().get(3).getAnswerList().get(1).select();
		assertTrue(Dialog.getNextQuestion().equals(sd1.getQuestionList().get(4)));
		assertEquals(new BooleanValue(false),Dialog.getAttribute("transportation").getValue());
		
		sd1.getQuestionList().get(4).getAnswerList().get(1).select();
		assertTrue(Dialog.getNextQuestion().equals(sd1.getQuestionList().get(5)));
		assertEquals(new BooleanValue(false),Dialog.getAttribute("limousine").getValue());
	}
	
	@Test
	public void testMovesToNextSubDialog() {
		sd1.getQuestionList().get(5).getAnswerList().get(1).select();
		assertTrue(Dialog.getNextQuestion().equals(sd2.getQuestionList().get(0)));
		assertEquals(new BooleanValue(false),Dialog.getAttribute("localKnowledge").getValue());
	}
	
	/** first question w/o a trigger*/
	@Test
	public void testGetNextSequentialQuestion() {
		sd2.getQuestionList().get(0).getAnswerList().get(0).select();
		assertTrue(Dialog.getNextQuestion().equals(sd2.getQuestionList().get(1)));
		assertEquals(new StringValue("water"),Dialog.getAttribute("drink").getValue());
	}
	
	@Test
	public void testFinishReturnsNull() {
		sd2.getQuestionList().get(1).getAnswerList().get(0).select();
		assertNull(Dialog.getNextQuestion());
		assertEquals(new BooleanValue(true),Dialog.getAttribute("likedIt").getValue());
	}
	
	@Test
	public void testResultingRecommendations() {
		PriorityQueue<RecommendationResult> recs = Dialog.getResultingRecommendations();
		assertEquals(3, recs.size());
		RecommendationResult rec = recs.poll();
		assertEquals("Go out with Profesor Pollice",rec.getOriginalRec().getName());
		assertEquals(105,rec.getConfidence());
		rec = recs.poll();
		assertEquals("City Bar Guide",rec.getOriginalRec().getName());
		assertEquals(100,rec.getConfidence());
		rec = recs.poll();
		assertEquals("Fancy Restaurant",rec.getOriginalRec().getName());
		assertEquals(25,rec.getConfidence());
	}
	
	
	@SuppressWarnings("serial")
	public static Database dialogDatabase() {
		AttributeReference attrRef = new AttributeReference();
		attrRef.addAttribute(new AttributeDescription("hungryOrThirsty",ValueType.STRING));
		attrRef.addAttribute(new AttributeDescription("21+",ValueType.BOOLEAN));
		attrRef.addAttribute(new AttributeDescription("mealType",ValueType.STRING));
		attrRef.addAttribute(new AttributeDescription("transportation",ValueType.BOOLEAN));
		attrRef.addAttribute(new AttributeDescription("limousine",ValueType.BOOLEAN));
		attrRef.addAttribute(new AttributeDescription("localKnowledge",ValueType.BOOLEAN));
		attrRef.addAttribute(new AttributeDescription("random",ValueType.BOOLEAN));
		attrRef.addAttribute(new AttributeDescription("drink",ValueType.STRING));
		attrRef.addAttribute(new AttributeDescription("likedIt",ValueType.BOOLEAN));
		
		Collection<Recommendation> recs = new ArrayList<Recommendation>();
		

		List<Scenario> recProps = 
				new ArrayList<Scenario>() {{
			add(new Scenario(new Attribute("21+",new BooleanValue(true)),100,true));
		}};
		
		List<Asset> recAssets = 
				new ArrayList<Asset>() {{
			add(new AssetStore(AssetType.LINK,new HashMap<String,String>() {{
				put("url","http://www.boston.com/thingstodo/nightlife/");
				put("text", "Bar Guide");
			}}));
		}};
		
		recs.add(new BasicRecommendation("City Bar Guide",recProps,recAssets,"A guide to bars in the city"));
		
		recProps = 
				new ArrayList<Scenario>() {{
			add(new Scenario(new Attribute("mealType",new StringValue("formal dinner")),25,false));
			add(new Scenario(new Attribute("limousine",new BooleanValue(true)),20,false));
			add(new Scenario(new Attribute("hungryOrThirsty",new StringValue("hungry")),22,false));
		}};
		
		recAssets = 
				new ArrayList<Asset>() {{
			add(new AssetStore(AssetType.TEXT,new HashMap<String,String>() {{
				put("text","Fancy Restaurants are great places to eat if you want food, and a formal, linousine friendly experience");
			}}));
			add(new AssetStore(AssetType.LINK,new HashMap<String,String>() {{
				put("url","http://www.bostonmagazine.com/articles/2011/11/the-50-best-restaurants-2011/");
				put("text","50 Best Restaurants in Boston");
			}}));
		}};
		
		recs.add(new BasicRecommendation("Fancy Restaurant",recProps,recAssets,"A fancy restaurant"));
		
		recProps = 
				new ArrayList<Scenario>() {{
			add(new Scenario(new Attribute("likedIt",new BooleanValue(true)),95,true));
			add(new Scenario(new Attribute("21+",new BooleanValue(true)),10,false));
		}};
		
		recAssets = 
				new ArrayList<Asset>() {{
			add(new AssetStore(AssetType.LINK,new HashMap<String,String>() {{
				put("url","http://web.cs.wpi.edu/~gpollice/");
				put("text","Professor Pollice's Schedule");
			}}));
			add(new AssetStore(AssetType.ONLINE_IMAGE,new HashMap<String,String>() {{
				put("src","http://images3.wikia.nocookie.net/__cb20060212075613/looneytunes/images/f/fc/Daffy_duck.gif");
				put("alt","Daffy Duck");
				put("caption","Professor Pollice likes ducks");
			}}));
		}};
		
		recs.add(new BasicRecommendation("Go out with Profesor Pollice",recProps,recAssets,"A great time"));
		return new TestDatabase(recs,attrRef);
	}
	

}
