package dsl.antlr;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.List;
import java.util.Map;

import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.tree.CommonTreeNodeStream;
import org.junit.Test;

import recommendation.RecommendationBuilder;
import recommendation.Scenario;
import recommendation.asset.Asset;
import recommendation.asset.AssetType;
import value.BooleanValue;
import value.IntegerValue;
import value.StringValue;
import attribute.Attribute;
import dsl.SpcAST;
import dsl.SpcASTAdaptor;

public class DSLRecCreatorTests {

	@Test
	public void testDescription() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Description: This is a test description\nto make sure everything is working well";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		assertEquals("RecName",recBuilder.getName());
		assertEquals("This is a test description\nto make sure everything is working well",recBuilder.getDescription());
	}
	
	@Test
	public void testSummary() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Summary: This is a test summary\n\t to make sure everything is working.";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		assertEquals("RecName",recBuilder.getName());
		assertEquals("This is a test summary\n\t to make sure everything is working.",recBuilder.getSummary());
	}
	
	@Test
	public void testSimpleScenario() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given distance is 5\n" +
					  "			Then Add 4 confidence\n";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(4, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		assertEquals(new Attribute("distance",new IntegerValue(5)),s.getAttributes().get(0));
		assertFalse(s.isRequired() || s.shouldStop() || s.isNotRecommended());
	}
	
	@Test
	public void testScenarioSpecialActions() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given distance is 5\n" +
					  "			Then Make Required\n" +
					  "			   And Do not recommend\n" +
					  "			   And Stop";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(0, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		assertEquals(new Attribute("distance",new IntegerValue(5)),s.getAttributes().get(0));
		assertTrue(s.isRequired());
		assertTrue(s.shouldStop());
		assertTrue(s.isNotRecommended());
	}
	
	@Test
	public void testScenarioRemoveAction() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given distance is 5\n" +
					  "			Then Remove 10 confidence\n";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(-10, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		assertEquals(new Attribute("distance",new IntegerValue(5)),s.getAttributes().get(0));
		assertFalse(s.isRequired() || s.shouldStop() || s.isNotRecommended());
	}
	
	@Test
	public void testScenarioBetweenOperator() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given distance is between 10 and 15\n" +
					  "			Then Add 4 confidence\n";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(4, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		assertEquals(new Attribute("distance",new IntegerValue(10,15)),s.getAttributes().get(0));
		assertFalse(s.isRequired() || s.shouldStop() || s.isNotRecommended());
	}
	
	@Test
	public void testScenarioBooleanValue() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given government = false\n" +
					  "			Then Add 4 confidence\n";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(4, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		assertEquals(new Attribute("government",new BooleanValue(false)),s.getAttributes().get(0));
		assertFalse(s.isRequired() || s.shouldStop() || s.isNotRecommended());
	}
	
	@Test
	public void testOneCategory() throws Exception {
		String text = "Recommendation: OneCat\n" +
					  "		Category: Source Control";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<String> categories = recBuilder.getCategories();
		assertEquals(1,categories.size());
		assertEquals("Source Control",categories.get(0));
	}
	
	@Test
	public void testTwoCategories() throws Exception {
		String text = "Recommendation: OneCat\n" +
					  "		Category: Source Control\n" +
					  "     Category: Tools";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<String> categories = recBuilder.getCategories();
		assertEquals(2,categories.size());
		assertEquals("Source Control",categories.get(0));
		assertEquals("Tools",categories.get(1));
	}
	
	@Test
	public void testAsset() throws Exception {
		String text = "Recommendation: AssetRec\n" +
					  "		Link Asset:\n" +
					  "			URL: http://wpi.edu/\n" +
					  "			Text: A great college";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Asset> assets = recBuilder.getAssets();
		assertEquals(1, assets.size());
		Asset asset = assets.get(0);
		assertEquals(AssetType.LINK,asset.getType());
		Map<String,String> assetContent = asset.getContents();
		assertTrue(assetContent.keySet().contains("url"));
		assertTrue(assetContent.keySet().contains("text"));
		assertEquals("http://wpi.edu/", assetContent.get("url"));
		assertEquals("A great college",assetContent.get("text"));
	}
	
	@Test
	public void testScenarioWithAsset() throws Exception {
		String text = "Recommendation: AssetRec\n" +
					  "		Scenario:\n" +
					  "        Given college = wpi" +
					  "		   Then Add {" +
					  "		Link Asset:\n" +
					  "			URL: http://wpi.edu/\n" +
					  "			Text: A great college}";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		//test scenario
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(0, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		assertEquals(new Attribute("college",new StringValue("wpi")),s.getAttributes().get(0));
		assertFalse(s.isRequired() || s.shouldStop() || s.isNotRecommended());
		//test asset inside scenario
		List<Asset> assets = s.getAssets();
		assertEquals(1, assets.size());
		Asset asset = assets.get(0);
		assertEquals(AssetType.LINK,asset.getType());
		Map<String,String> assetContent = asset.getContents();
		assertTrue(assetContent.keySet().contains("url"));
		assertTrue(assetContent.keySet().contains("text"));
		assertEquals("http://wpi.edu/", assetContent.get("url"));
		assertEquals("A great college",assetContent.get("text"));
	}
	
	@Test
	public void testScenarioWithExpressionOr() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given distance is 5 or > 6 or <= 3\n" +
					  "			Then Add 4 confidence\n";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(4, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		IntegerValue value = new IntegerValue(5);
		value.addGreaterThan(6);
		value.addLessThan(3,true);
		assertEquals(new Attribute("distance",value),s.getAttributes().get(0));
		assertFalse(s.isRequired() || s.shouldStop() || s.isNotRecommended());
	}
	
	@Test
	public void testScenarioWithExpressionOrStrings() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given college = wpi or != mit or equals \"rpi\"\n" +
					  "			Then Add 4 confidence\n";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(4, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		StringValue value = new StringValue("wpi");
		value.addNotString("mit");
		value.addString("rpi");
		Attribute actual = s.getAttributes().get(0);
		assertEquals(new Attribute("college",value),actual);
		assertFalse(s.isRequired() || s.shouldStop() || s.isNotRecommended());
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void testScenarioWithExpressionOrBoolean() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given government is true or is false\n" +
					  "			Then Add 4 confidence\n";
		makeRecCreator(text);
	}
	
	@Test
	public void testScenarioWithExpressionNotBoolean() throws Exception {
		String text = "Recommendation: RecName\n" +
					  "		Scenario:\n" +
					  "			Given government != true\n" +
					  "			Then Add 4 confidence\n";
		RecommendationBuilder recBuilder = makeRecCreator(text);
		List<Scenario> scenarios = recBuilder.getScenarios();
		assertEquals(1,scenarios.size());
		Scenario s = scenarios.get(0);
		assertEquals(4, s.getWeight());
		assertEquals(1,s.getAttributes().size());
		BooleanValue value = new BooleanValue(false);
		Attribute actual = s.getAttributes().get(0);
		assertEquals(new Attribute("government",value),actual);
		assertFalse(s.isRequired() || s.shouldStop() || s.isNotRecommended());
	}
	
	private DSLParser makeParser(Reader reader) {
		try {
			DSLParser p = new DSLParser(
					new CommonTokenStream(
							new DSLLexer(
									new ANTLRReaderStream(
											reader))));
			p.setTreeAdaptor(new SpcASTAdaptor());
			return p;
		} catch (IOException e) {
			throw new RuntimeException("IOException while creating a parser: " + e.getMessage());
		}
	}
	
	
	private RecommendationBuilder makeRecCreator(Reader reader) throws Exception
	{
		SpcAST tree = (SpcAST)makeParser(reader).recommendation().getTree();
		DSLRecCreator recCreator = new DSLRecCreator(new CommonTreeNodeStream(new SpcASTAdaptor(),tree));
		recCreator.setTreeAdaptor(new SpcASTAdaptor());
		tree = (SpcAST) recCreator.recommendation().getTree();
		return recCreator.getRecBuilder();
	}

	private RecommendationBuilder makeRecCreator(String text) throws Exception {
		return makeRecCreator(new StringReader(text));
	}
}
